#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC BT
"""
from __future__ import absolute_import
from datetime import datetime
import traceback
import math
import sys

from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import format_f, tallar_text, get_forced_elements, adapt_diff
from libcnmc.models import F2Res4666

QUIET = False


class LBT(StopMultiprocessBased):
    """
    Class that generates the LBT(2) file of the 4666
    """
    def __init__(self, **kwargs):
        """
        Class constructor
        :param kwargs: year(generation year), codi_r1 R1 code
        :return: CT
        """
        super(LBT, self).__init__(**kwargs)
        self.year = kwargs.pop("year", datetime.now().year - 1)
        self.codi_r1 = kwargs.pop("codi_r1")
        self.base_object = "Línies BT"
        self.report_name = "CNMC INVENTARI BT"
        self.embarrats = kwargs.pop("embarrats", False)
        self.compare_field = kwargs["compare_field"]
        self.extended = kwargs.get("extended", False)
        self.prefix = kwargs.pop('prefix', 'B') or 'B'

    def get_sequence(self):
        """
        Method that generates a list of ids to pass to the consummer
        :return: List of ids
        :rtype: list(int)
        """
        data_pm = '{0}-01-01'.format(self.year + 1)
        data_baixa = '{0}-01-01'.format(self.year)
        search_params = []
        if not self.embarrats:
            search_params += [('cable.tipus.codi', '!=', 'E')]
        search_params += [('propietari', '=', True),
                          ('data_pm', '<', data_pm),
                          '|',
                          '&', ('data_baixa', '>', data_baixa),
                               ('baixa', '=', True),
                          '|',
                               ('data_baixa', '=', False),
                               ('baixa', '=', False)
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        ids = self.connection.GiscedataBtElement.search(
            search_params, 0, 0, False, {'active_test': False})

        forced_ids = get_forced_elements(
            self.connection,
            "giscedata.bt.element"
        )

        ids = ids + forced_ids["include"]
        ids = list(set(ids) - set(forced_ids["exclude"]))

        return list(set(ids))

    def consumer(self):
        """
        Method that generates the csv file
        :return: List of arrays
        """
        O = self.connection
        count = 0
        fields_to_read = [
            'name', 'municipi', 'data_pm', 'ct', 'coeficient', 'cini',
            'perc_financament', 'longitud_cad', 'cable', 'voltatge',
            'data_alta', 'propietari', 'tipus_instalacio_cnmc_id', 'baixa',
            'data_baixa', "edge_id", self.compare_field, 'id_regulatori'
        ]
        data_pm_limit = '{0}-01-01'.format(self.year + 1)
        while True:
            try:
                count += 1
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)

                linia = O.GiscedataBtElement.read(item, fields_to_read)

                # Si hi ha 'id_regulatori' el posem
                if linia.get('id_regulatori', False):
                    o_linia = linia['id_regulatori']
                else:
                    o_linia = '{}{}'.format(self.prefix, linia['name'])

                comunitat = ''
                if linia['municipi']:
                    ccaa_obj = O.ResComunitat_autonoma
                    id_comunitat = ccaa_obj.get_ccaa_from_municipi(
                        linia['municipi'][0])
                    id_comunitat = id_comunitat[0]
                    comunidad = ccaa_obj.read(id_comunitat, ['codi'])
                    if comunidad:
                        comunitat = comunidad['codi']
                data_pm = ''
                if linia['data_pm']:
                    data_pm_linia = datetime.strptime(str(linia['data_pm']),
                                                      '%Y-%m-%d')
                    data_pm = data_pm_linia.strftime('%d/%m/%Y')
                data_baixa = ''
                if linia['baixa'] and linia['data_baixa']:
                    data_baixa = datetime.strptime(str(linia['data_baixa']),
                                                      '%Y-%m-%d')
                    data_baixa = data_baixa.strftime('%d/%m/%Y')

                # Coeficient per ajustar longituds de trams
                coeficient = linia['coeficient'] or 1.0

                try:
                    tensio = (int(linia['voltatge']) / 1000.0)
                except Exception:
                    tensio = 0.0

                if linia['tipus_instalacio_cnmc_id']:
                    id_ti = linia.get('tipus_instalacio_cnmc_id')[0]
                    codi_ccuu = O.GiscedataTipusInstallacio.read(
                        id_ti,
                        ['name'])['name']
                else:
                    codi_ccuu = ''

                # Agafem el cable de la linia
                if linia['cable']:
                    cable = O.GiscedataBtCables.read(linia['cable'][0], [
                        'intensitat_admisible', 'seccio'])
                else:
                    cable = {'seccio': 0, 'intensitat_admisible': 0}

                intensitat = cable['intensitat_admisible']
                # Capacitat
                capacitat = round(
                    (cable['intensitat_admisible'] * int(linia['voltatge'])
                     * math.sqrt(3)) / 1000, 3)

                if not capacitat:
                    capacitat = 1.0

                longitud = round(linia['longitud_cad'] * coeficient / 1000.0,
                                 3) or 0.001
                if linia['data_baixa']:
                    if linia['data_baixa'] > data_pm_limit:
                        fecha_baja = ''
                    else:
                        tmp_date = datetime.strptime(
                            linia['data_baixa'], '%Y-%m-%d')
                        fecha_baja = tmp_date.strftime('%d/%m/%Y')
                else:
                    fecha_baja = ''
                last_data = {}
                if linia[self.compare_field]:
                    last_data = linia[self.compare_field]
                    entregada = F2Res4666(**last_data)
                    origen = last_data["origen"]
                    final = last_data["destino"]
                    actual = F2Res4666(
                        '{}{}'.format(self.prefix, linia['name']),
                        linia['cini'],
                        origen or '',
                        final or '',
                        codi_ccuu or '',
                        comunitat,
                        comunitat,
                        format_f(
                            100.0 - linia.get('perc_financament', 0.0), 2
                        ),
                        data_pm,
                        data_baixa,
                        1,
                        1,
                        format_f(tensio, 3),
                        format_f(longitud, 3),
                        format_f(intensitat),
                        format_f(float(cable['seccio']), 2),
                        format_f(capacitat, 3),
                        0
                    )
                    if actual == entregada and fecha_baja == '':
                        estado = 0
                    else:
                        self.output_m.put("{} {}".format(linia["name"], adapt_diff(actual.diff(entregada))))
                        estado = 1
                else:
                    if linia['data_pm']:
                        if linia['data_pm'][:4] != str(self.year):
                            self.output_m.put("Identificador:{} No estava en el fitxer carregat al any n-1 i la data de PM es diferent al any actual".format(linia["name"]))
                            estado = '1'
                        else:
                            estado = '2'
                    else:
                        self.output_m.put("Identificador:{} No estava en el fitxer carregat al any n-1".format(linia["name"]))
                        estado = '1'
                if linia.get("edge_id", False):
                    edge_id =  linia["edge_id"][0]
                    edge = O.GiscegisEdge.read(edge_id, ['start_node','end_node'])
                    origen = edge['start_node'][1]
                    final = edge['end_node'][1]
                else:

                    edge_id = O.GiscegisEdge.search(
                        [
                            ('id_linktemplate', '=', linia['name']),
                            ('layer', 'ilike', '%BT%')
                        ]
                    )
                    if edge_id:
                        edge = O.GiscegisEdge.read(
                            edge_id[0], ['start_node', 'end_node']
                        )
                        origen = edge['start_node'][1]
                        final = edge['end_node'][1]
                    else:
                        # El tram ha estat donat de baixa aquest any i ja no
                        # te edge. Utilitzem el seu propi nom com a nus inicial
                        # i nus final
                        origen = str(linia['name']) + '_0'
                        final = str(linia['name']) + '_1'

                output = [
                    o_linia,  # IDENTIFICADOR
                    linia['cini'] or '',            # CINI
                    origen or '',                   # ORIGEN
                    final or '',                    # DESTINO
                    codi_ccuu or '',                # CODIGO_CCUU
                    comunitat,                      # CODIGO_CCAA
                    comunitat,                      # CODIGO_CCAA_2
                    format_f(
                        100.0 - linia.get('perc_financament', 0.0), 2
                    ),                              # FINANCIADO
                    data_pm or '',                  # FECHA_APS
                    fecha_baja,                     # FECHA_BAJA
                    1,                              # NUMERO CIRCUITOS
                    1,                              # NUMERO CONDUCTORES
                    format_f(tensio, 3),            # NIVEL_TENSION
                    format_f(longitud, 3),          # LONGITUD
                    format_f(intensitat, 3),        # INTENSITAT MAXIMA
                    format_f(cable['seccio'], 3),   # SECCION
                    format_f(capacitat, 3),         # CAPACIDAD
                    estado                          # ESTADO
                ]

                if self.extended:

                    if 'municipi' in linia:
                        if linia['municipi']:
                            municipi = O.ResMunicipi.read(
                                linia['municipi'][0], ['name']
                            )
                            output.append(municipi.get('name', ""))
                        else:
                            output.append("")
                    else:
                        output.append("")

                    if 'ct' in linia:
                        if linia['ct']:
                            ct = O.GiscedataCts.read(linia['ct'][0], ['zona_id'])
                            if 'zona_id' in ct:
                                if ct['zona_id']:
                                    zona = O.GiscedataCtsZona.read(
                                        ct['zona_id'][0], ['name']
                                    )
                                    output.append(zona.get('name', ""))
                                else:
                                    output.append("")
                            else:
                                output.append("")
                        else:
                            output.append("")
                    else:
                        output.append("")

                self.output_q.put(output)
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



# -*- coding: utf-8 -*-
from multiprocessing import Manager


from libcnmc.core import UpdateFile

class CreateCelles(UpdateFile):
    def __init__(self, **kwargs):
        super(CreateCelles, self).__init__(**kwargs)
        self.header = [
            'name', 'tipus_element', 'installacio', 'tipus_posicio',
            'inventari', 'aillament', 'cini', 'propietari', 'perc_financament',
            'tensio'
        ]
        self.search_keys = [('name')]
        self.fields_read_ct = ['perc_financament', 'propietari']
        self.fields_read_at_tram = ['perc_financament']
        self.object = self.connection.GiscedataCellesCella
        manager = Manager()
        self.cts = manager.dict()
        self.at_suports = manager.dict()


    def model2cc(self, model):
        """Converteix el model en CamelCase.
        """
        return ''.join(map(lambda x: x.capitalize(), model.split('.')))

    def search_and_update(self, vals):
        id_cella = self.object.create(vals)

    def get_value(self, tipus, clau, camp):
        if tipus == 'ct':
            if self.cts.get(clau, {}).get(camp, False):

                return self.cts[clau][camp]
            else:
                ct_dades = self.connection.GiscedataCts.read(
                    self.cts[clau]['id'], self.fields_read_ct)
                vals = ct_dades.copy()
                vals.update(self.cts[clau])
                self.cts[clau] = vals
                return self.cts[clau][camp]
        elif tipus == 'suport':
            if self.at_suports.get(clau, {}).get(camp, False):
                return self.at_suports[clau][camp]
            else:
                suport_dades = self.connection.GiscedataAtSuport.read(
                    self.at_suports[clau]['id'], ['linia'])
                linia_id = suport_dades['linia'][0]
                linia_dades = self.connection.GiscedataAtLinia.read(
                    linia_id, ['trams', 'propietari']
                )
                tram_dades = self.connection.GiscedataAtTram.read(
                    linia_dades['trams'][0], self.fields_read_at_tram
                )
                vals = tram_dades.copy()
                vals.update({
                    'propietari': linia_dades['propietari']
                })
                vals.update(self.at_suports[clau])
                self.at_suports[clau] = vals
                return self.at_suports[clau][camp]

    def build_vals(self, values):
        vals = {}
        o = self.connection
        ct_name = False
        suport_name = False
        for val in zip(self.header, values):
            if val[0] == 'tipus_posicio':
                search_param = [('codi', '=', val[1])]
                value = o.GiscedataCellesTipusPosicio.search(search_param)[0]
                vals[val[0]] = value
            elif val[0] == 'tipus_element':
                search_param = [('codi', '=', val[1])]
                value = o.GiscedataCellesTipusElement.search(search_param)[0]
                vals[val[0]] = value
            elif val[0] == 'aillament':
                search_param = [('name', '=', val[1].upper())]
                value = o.GiscedataCellesAillament.search(search_param)[0]
                vals[val[0]] = value
            elif val[0] == 'tensio':
                search_param = [('name', '=', val[1].upper())]
                value = o.GiscedataTensionsTensio.search(search_param)[0]
                vals[val[0]] = value
            elif val[0] == 'installacio':
                model, name = val[1].split(',')
                model = model.lower()
                if 'ct' in model:
                    if name in self.cts:
                        ct_id = self.cts[name]['id']
                    else:
                        ct_id = o.GiscedataCts.search(
                            [('name', '=', name)],
                            0, 0, False,
                            {'active_test': False})[0]
                        self.cts[name] = {'id': ct_id}
                    value = '%s,%s' % ('giscedata.cts', ct_id)
                    ct_name = name
                elif 'suport' in model:
                    if name in self.at_suports:
                        at_suport_id = self.at_suports[name]['id']
                    else:
                        at_suport_id = o.GiscedataAtSuport.search(
                            [('name', '=', name)], 0, 0, False,
                            {'active_test': False})[0]
                        self.at_suports[name] = {'id': at_suport_id}
                    value = '%s,%s' % ('giscedata.at.suport', at_suport_id)
                    suport_name = name
                vals[val[0]] = value
            elif val[0] == 'propietari':
                if val[1] == 'auto':
                    if ct_name:
                        tipus = 'ct'
                        name = ct_name
                    if suport_name:
                        tipus = 'suport'
                        name = suport_name
                    vals[val[0]] = self.get_value(tipus, name, 'propietari')
                else:
                    vals[val[0]] = int(val[1])
            elif val[0] == 'perc_financament':
                if val[1] == 'auto':
                    if ct_name:
                        tipus = 'ct'
                        name = ct_name
                    if suport_name:
                        tipus = 'suport'
                        name = suport_name
                    vals[val[0]] = self.get_value(
                        tipus, name, 'perc_financament')
                else:
                    vals[val[0]] = float(val[1])
            else:
                vals[val[0]] = val[1]
        return vals




# -*- coding: utf-8 -*-
import click
import os
import re
import time

_VERSION = 7.1

_F1 = [
    'Node',
    'Coordenada X',
    'Coordenada Y',
    'Coordenada Z',
    'CNAE',
    'Equip de mesura',
    'Codi de tarifa',
    'Zona de qualitat',
    'CUPS',
    'Codi distribuidora',
    'Municipi',
    'Provincia',
    'Connexió',
    'Tensió d\'alimentació',
    'Estat del contracte',
    'Potència contractada',
    'Potència facturada',
    'Potència adscrita',
    'Energia anual activa',
    'Energia anual reactiva',
    'Any d\'informació'
]

_F1_exclude = [
    0, 1, 2, 3, 12, 19
]

_F1BIS = [
    'CUPS',
    'CINI',
    'Instal·lació',
    'Número de lectures',
    'Canvi de titularitat',
    'Baixa del subministre',
    'Any d\'informació'
]

_F2A = [
    'Codi de mercat',
    'Codi companyia distribuidora declarant',
    'Codi companyia distribuidora client',
    'Codi frontera REE',
    'Coordenada X',
    'Coordenada Y',
    'Tensió'
]

_F2B = [
    'Codi de mercat',
    'Codi companyia distribuidora declarant',
    'Municipi',
    'Provincia'
]

_F10 = [
    'Tram',
    'Node inicial',
    'Node final',
    'CINI',
    'Provincia',
    'Nivell de tensió',
    'Longitud',
    'Número de circuits',
    'Tipus',
    'R',
    'X',
    'Intensitat màxima',
    'Estat operació',
    'Codi companyia distribuidora',
    'Propietat',
    'Any d\'informació'
]


@click.command()
@click.option('-d', '--dir', help='Ruta de la carpeta amb els formularis')
@click.option('-l', '--lang', help='Idioma amb el qual es generaràn les '
                                   'validacions (es_ES o ca_ES)')
def node_check(**kwargs):
    if not kwargs['dir']:
        print "ERROR: falta especificar la ruta de la carpeta amb els" \
              " formularis."
    elif not kwargs['lang']:
        print "ERROR: falta especificar l'idioma (es_ES o ca_ES)"
    else:
        ruta = kwargs['dir']
        lang = re.sub('[^A-Za-z0-9_]+', '', kwargs["lang"])
        if os.path.exists(ruta):
            forms = os.listdir(ruta)
            forms.sort()
            buidar_resultats_anteriors(ruta, lang)
            f11 = None
            f10 = None
            f9 = None
            f1 = None
            for form in forms:
                if '_11_' in form and '~' not in form:
                    f11 = ruta+'/'+form
                if '_10_' in form and '~' not in form:
                    f10 = ruta+'/'+form
                if '_9_' in form and '~' not in form:
                    f9 = ruta+'/'+form
                if '_1_' in form and '~' not in form:
                    f1 = ruta+'/'+form
            if f10:
                for form in forms:
                    ruta_completa = ruta+'/'+form
                    if '_1_' in form and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando nodos del formulario 1...\n"
                        else:
                            print "Verificant nodes del formulari 1...\n"
                        nodes = check(f10, ruta_completa, 'f1')
                        mostrar_nodes(nodes, 'f1', ruta, lang)
                        if lang == 'es_ES':
                            print "Verificando CUPS del formulario 1...\n"
                        else:
                            print "Verificant CUPS del formulari 1...\n"
                        cups = check_cups(ruta_completa, 'f1')
                        mostrar_cups(cups, 'f1', ruta, lang)
                        if lang == 'es_ES':
                            print "Verificando campos obligatorios del " \
                                  "formulario 1...\n"
                        else:
                            print "Verificant camps obligatoris del " \
                                  "formulari 1...\n"
                        camps = check_obligatoris(ruta_completa, 'f1')
                        mostrar_obligatoris(camps, 'f1', ruta, lang)
                    elif ('_1bis_' in form or '_1BIS_' in form) \
                            and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando CUPS del formulario 1bis...\n"
                        else:
                            print "Verificant CUPS del formulari 1bis...\n"
                        cups = check_cups(ruta_completa, 'f1bis')
                        mostrar_cups(cups, 'f1bis', ruta, lang)
                        cups = check_cups_f1bis(ruta_completa, f1)
                        mostrar_cups_f1bis(cups, ruta, lang)
                        if lang == 'es_ES':
                            print "Verificando campos obligatorios del " \
                                  "formulario 1bis...\n"
                        else:
                            print "Verificant camps obligatoris del " \
                                  "formulari 1bis...\n"
                        camps = check_obligatoris(ruta_completa, 'f1bis')
                        mostrar_obligatoris(camps, 'f1bis', ruta, lang)
                    elif '_2_' in form and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando nodos del formulario 2...\n"
                        else:
                            print "Verificant nodes del formulari 2...\n"
                        nodes = check(f10, ruta_completa, 'f2')
                        mostrar_nodes(nodes, 'f2', ruta, lang)
                        if lang == 'es_ES':
                            print "Verificando CUPS del formulario 2...\n"
                        else:
                            print "Verificant CUPS del formulari 2...\n"
                        cups = check_cups(ruta_completa, 'f2')
                        mostrar_cups(cups, 'f2', ruta, lang)
                    elif ('_2a_' in form or '_2A_' in form) and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando campos obligatorios del " \
                                  "formulario 2a...\n"
                        else:
                            print "Verificant camps obligatoris del " \
                                  "formulari 2a..."
                        camps = check_obligatoris(ruta_completa, 'f2a')
                        mostrar_obligatoris(camps, 'f2a', ruta, lang)
                    elif ('_2b_' in form or '_2B_' in form) and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando campos obligatorios del " \
                                  "formulario 2b...\n"
                        else:
                            print "Verificant camps obligatoris del " \
                                  "formulari 2b..."
                        camps = check_obligatoris(ruta_completa, 'f2b')
                        mostrar_obligatoris(camps, 'f2b', ruta, lang)
                    elif '_3_' in form and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando nodos del formulario 3...\n"
                        else:
                            print "Verificant nodes del formulari 3...\n"
                        nodes = check(f10, ruta_completa, 'f3')
                        mostrar_nodes(nodes, 'f3', ruta, lang)
                    elif '_6_' in form and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando nodos del formulario 6...\n"
                        else:
                            print "Verificant nodes del formulari 6...\n"
                        nodes = check(f10, ruta_completa, 'f6')
                        mostrar_nodes(nodes, 'f6', ruta, lang)
                        if lang == 'es_ES':
                            print "Verificando CUPS del formulario 6...\n"
                        else:
                            print "Verificant CUPS del formulari 6...\n"
                        cups = check_cups(ruta_completa, 'f6')
                        mostrar_cups(cups, 'f6', ruta, lang)
                    elif '_9_' in form and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando tramos de los formularios" \
                                  "9 y 10...\n"
                        else:
                            print "Verificant trams dels formularis 9 i 10...\n"
                        trams, topologia = check_trams_f9_f10(
                            f10, ruta_completa)
                        mostrar_trams(trams, ruta, 'f9_f10', lang, topologia)
                        if lang == 'es_ES':
                            print "Verificando trams repetidos del " \
                                  "formulario 9...\n"
                        else:
                            print "Verificant trams repetits del formulari " \
                                "9...\n"
                        repetits = comprovar_repetits(ruta_completa, 'f9')
                        mostrar_trams(repetits, ruta, 'f9r', lang)
                    elif '_10_' in form and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando tramos repetidos del " \
                                  "formulario 10...\n"
                        else:
                            print "Verificant trams repetits del formulari 10...\n"
                        repetits = comprovar_repetits(ruta_completa, 'f10')
                        mostrar_trams(repetits, ruta, 'f10r', lang)
                        if lang == 'es_ES':
                            print "Verificando campos obligatorios del " \
                                  "formulario 10...\n"
                        else:
                            print "Verificant camps obligatoris del formulari " \
                                  "10...\n"
                        camps = check_obligatoris(ruta_completa, 'f10')
                        mostrar_obligatoris(camps, 'f10', ruta, lang)
                    elif '_11_' in form and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando nodos del formulario 11...\n"
                        else:
                            print "Verificant nodes del formulari 11...\n"
                        nodes = check(f10, ruta_completa, 'f11')
                        mostrar_nodes(nodes, 'f11', ruta, lang)
                    elif '_12_' in form and '~' not in form:
                        if f11:
                            if lang == 'es_ES':
                                print "Verificando CTs del formulario 12...\n"
                            else:
                                print "Verificant CTs del formulari 12...\n"
                            cts = comprovar_cts(f11, ruta_completa, 'f12')
                            mostrar_cts(ruta, cts, 'f12', lang)
                    elif ('_12bis_' in form or '_12BIS_' in form) \
                            and '~' not in form:
                        if f11:
                            if lang == 'es_ES':
                                print "Verificando CTs del formulario 12bis...\n"
                            else:
                                print "Verificant CTs del formulari 12bis...\n"
                            cts = comprovar_cts(f11, ruta_completa, 'f12bis')
                            mostrar_cts(ruta, cts, 'f12bis', lang)
                            if lang == 'es_ES':
                                print "Verificando códigos de posición del " \
                                      "formulario 12bis..\n"
                            else:
                                print "Verificant codis de posició del " \
                                      "formulari 12bis...\n"
                        posicions = comprovar_repetits(ruta_completa, 'f12bis')
                        mostrar_posicions(ruta, posicions, lang)
                    elif ('_13bis_' in form or '_13BIS_' in form) \
                            and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando nodos del formulario 13bis...\n"
                        else:
                            print "Verificant nodes del formulari 13bis...\n"
                        nodes = check(f10, ruta_completa, 'f13bis')
                        mostrar_nodes(nodes, 'f13bis', ruta, lang)
                    elif '_15_' in form and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando nodos del formulario 15...\n"
                        else:
                            print "Verificant nodes del formulari 15...\n"
                        nodes = check(f10, ruta_completa, 'f15')
                        mostrar_nodes(nodes, 'f15', ruta, lang)
                        if f9:
                            print "Verificant trams del formulari 15 amb els" \
                                  " del formulari 9...\n"
                            trams = check_trams_f15(f9, ruta_completa)
                            mostrar_trams(trams, ruta, 'f15', lang)
                    elif '_16_' in form and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando nodos del formulario 16...\n"
                        else:
                            print "Verificant nodes del formulari 16...\n"
                        nodes = check(f10, ruta_completa, 'f16')
                        mostrar_nodes(nodes, 'f16', ruta, lang)
                    elif '_17_' in form and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando nodos del formulario 17...\n"
                        else:
                            print "Verificant nodes del formulari 17...\n"
                        nodes = check(f10, ruta_completa, 'f17')
                        mostrar_nodes(nodes, 'f17', ruta, lang)
                    elif '_20_' in form and '~' not in form:
                        if lang == 'es_ES':
                            print "Verificando CUPS del formulario 20...\n"
                        else:
                            print "Verificant CUPS del formulari 20...\n"
                        cups = check_cups(ruta_completa, 'f20')
                        mostrar_cups(cups, 'f20', ruta, lang)
                if lang == 'es_ES':
                    print "\n\nVerificación completada."
                else:
                    print "\n\nVerificació completada."
            else:
                if lang == 'es_ES':
                    print "ERROR: no se ha encontrado el formulario f10 en la" \
                          "carpeta especificada."
                else:
                    print "ERROR: no s'ha trobat el formulari f10 a la" \
                          "carpeta especificada."
        else:
            if lang == 'es_ES':
                print "ERROR: No se ha encontrado la carpeta de formularios " \
                      "especificada."
            else:
                print "ERROR: No s'ha trobat la carpeta de formularis " \
                      "especificada."


def buidar_resultats_anteriors(ruta, lang):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'w') as f:
        if lang == 'es_ES':
            f.write(
                'Resultados de la comprovación de los formularios '
                'CNMC 4/2015\n\n')
            f.write('Versión: {0}\n\n'.format(_VERSION))
            f.write('Fecha de comprobación: {0}\n\n'.format(
                time.strftime("%d/%m/%Y")))
        else:
            f.write('Resultats de la comprovació dels formularis '
                    'CNMC 4/2015\n\n')
            f.write('Versió: {0}\n\n'.format(_VERSION))
            f.write('Data de comprovació: {0}\n\n'.format(
                time.strftime("%d/%m/%Y")))
        f.write('#######################################################'
                '\n\n\n\n')


def check_obligatoris(fitxer, context):
    camps = {}
    if existeix_fitxer(fitxer):
        with open(fitxer, 'r') as f:
            # j és el numero de linia del fitxer on s'ha trobat que falta un
            # camp
            j = 1
            for linia in f:
                linia = linia.rstrip()
                camps_linia = linia.split(';')
                i = 0
                for camp in camps_linia:
                    if context == 'f1':
                        cups = camps_linia[8]
                        if camp == '' and i not in _F1_exclude:
                            if cups not in camps.keys():
                                camps[cups] = []
                            camps[cups].append(_F1[i])
                    elif context == 'f1bis':
                        cups = camps_linia[0]
                        if camp == '':
                            if cups not in camps.keys():
                                camps[cups] = []
                            camps[cups].append(_F1BIS[i])
                    elif context == 'f2a':
                        if camp == '':
                            if j not in camps.keys():
                                camps[j] = []
                            camps[j].append(_F2A[i])
                    elif context == 'f2b':
                        if camp == '':
                            if j not in camps.keys():
                                camps[j] = []
                            camps[j].append(_F2B[i])
                    elif context == 'f10':
                        tram = camps_linia[0]
                        if camp == '':
                            if tram not in camps.keys():
                                camps[tram] = []
                            if tram == '':
                                if j not in camps.keys():
                                    camps[j] = []
                                camps[j].append(_F10[i])
                            else:
                                camps[tram].append(_F10[i])
                    i += 1
                j += 1
    return camps


def mostrar_obligatoris(camps, form, ruta, lang):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        if camps:
            if lang == 'es_ES':
                f.write("Camps obligatorios del formulario {0} "
                        "que faltan:\n".format(form))
            else:
                f.write("Camps obligatoris del formulari {0} "
                        "que falten:\n".format(form))
            i = 0
            for elem in camps:
                if len(camps[elem]) > 0:
                    f.write("\n")
                    if form == 'f2a' or form == 'f2b':
                        if lang == 'es_ES':
                            f.write("\tLínea: {0}\n\n".format(elem))
                        else:
                            f.write("\tLínia: {0}\n\n".format(elem))
                    elif form == 'f10':
                        if "Tram" in camps[elem]:
                            if lang == 'es_ES':
                                f.write("\tLínea: {0}\n\n".format(elem))
                            else:
                                f.write("\tLínia: {0}\n\n".format(elem))
                        else:
                            if lang == 'es_ES':
                                f.write("\tTramo: {0}\n\n".format(elem))
                            else:
                                f.write("\tTram: {0}\n\n".format(elem))
                    else:
                        f.write("\tCUPS: {0}\n\n".format(elem))
                    i += 1
                    for c in camps[elem]:
                        f.write("\t\t{0}\n".format(c))
            if lang == 'es_ES':
                f.write("\nTotal de líneas con campos sin completar en"
                        " {0}: {1}\n\n".format(form, i))
            else:
                f.write("\nTotal de línies amb camps sense emplenar a {0}: "
                        "{1}\n\n".format(form, i))
        else:
            if lang == 'es_ES':
                f.write("Todos los campos obligatorios del fichero {0} están"
                        " completos. OK!\n\n".format(form))
            else:
                f.write("Tots els camps obligatoris del fitxer {0} estàn"
                        " complets. OK!\n\n".format(form))


def mostrar_cups_f1bis(cups, ruta, lang):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        if cups:
            if lang == 'es_ES':
                f.write("CUPS del formulario F1 Bis que no aparecen en el"
                        " formulario F1:\n\n")
            else:
                f.write("CUPS del formulari F1 Bis que no apareixen al"
                        " formulari F1:\n\n")
            for elem in cups:
                f.write("\t{0}\n".format(elem))
            f.write("\nTotal CUPS: {0}\n\n".format(len(cups)))
        else:
            if lang == 'es_ES':
                f.write("Los cups del formulario F1 Bis coinciden con los"
                        " del formulario F1. OK!\n\n")
            else:
                f.write("Els cups del formulari F1 Bis coincideixen amb els"
                        " del formulari F1. OK!\n\n")


def check_cups_f1bis(fitxer, f1):
    cups_f1 = []
    cups_f1bis = []
    resultat = []
    if existeix_fitxer(fitxer):
        if existeix_fitxer(f1):
            with open(fitxer) as f1bis:
                for linia in f1bis.readlines():
                    cups_f1bis.append(linia.split(';')[0])
            with open(f1) as f:
                for linia in f.readlines():
                    cups_f1.append(linia.split(';')[8])
            resultat = set(cups_f1bis) - set(cups_f1)
            resultat = list(resultat)
    return resultat


def mostrar_cups(cups, form, ruta, lang):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        if cups:
            if lang == 'es_ES':
                f.write("CUPS duplicados del formulario {0}:\n".format(form))
            else:
                f.write("CUPS duplicats del formulari {0}:\n".format(form))
            i = 0
            for elem in cups:
                if len(cups[elem]) > 1:
                    f.write("\n")
                    if lang == 'es_ES':
                        f.write("\tCódigo: {0}\n\n".format(elem))
                    else:
                        f.write("\tCodi: {0}\n\n".format(elem))
                    i += 1
                    for c in cups[elem]:
                        f.write("\t\t{0}\n".format(c))
            if lang == 'es_ES':
                f.write("\nTotal CUPS duplicados en {0}: {1}\n\n".format(
                    form, i))
            else:
                f.write("\nTotal CUPS duplicats a {0}: {1}\n\n".format(
                    form, i))
        else:
            if lang == 'es_ES':
                f.write("No hay CUPS duplicados en el formulario {0}. "
                        "OK!\n\n".format(form))
            else:
                f.write("No hi ha CUPS duplicats al formulari {0}. "
                        "OK!\n\n".format(form))


def check_cups(fitxer, form):
    cups = {}
    cups_repetits = {}
    if existeix_fitxer(fitxer):
        with open(fitxer, 'r') as f:
            for linia in f.readlines():
                if form == 'f1':
                    cups_complet = linia.split(';')[8]
                elif form == 'f1bis':
                    cups_complet = linia.split(';')[0]
                elif form == 'f2':
                    cups_complet = linia.split(';')[5]
                elif form == 'f6':
                    cups_complet = linia.split(';')[6]
                elif form == 'f20':
                    cups_complet = linia.split(';')[1]
                cups_trim = cups_complet[:18]
                if cups_trim in cups.keys():
                    cups_repetits[cups_trim] = cups[cups_trim]
                    cups_repetits[cups_trim].append(cups_complet)
                else:
                    cups[cups_trim] = [cups_complet]
    return cups_repetits


def mostrar_posicions(ruta, posicions, lang):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        if posicions:
            if lang == 'es_ES':
                f.write("Posiciones duplicadas del formulario F12 Bis:\n\n")
            else:
                f.write("Posicions duplicades del formulari F12 Bis:\n\n")
            for posicio in posicions:
                f.write("\t{0}\n".format(posicio))
            if lang == 'es_ES':
                f.write("\nTotal posiciones duplicadas F12 Bis: {0}\n\n".format(
                    len(posicions)))
            else:
                f.write("\nTotal posicions duplicades F12 Bis: {0}\n\n".format(
                    len(posicions)))
        else:
            if lang == 'es_ES':
                f.write("Los códigos de posición del formulario F12 Bis no están "
                        "duplicados. OK!\n\n")
            else:
                f.write("Els codis de posició del formulari F12 Bis no estan "
                        "duplicats. OK!\n\n")


def mostrar_cts(ruta, cts, context, lang):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        if cts:
            if lang == 'es_ES':
                f.write("CTs del formulario {0} que no aparecen en formulario "
                        "11:\n\n".format(context))
            else:
                f.write("CTs del formulari {0} que no apareixen al formulari "
                        "11:\n\n".format(context))
            for ct in cts:
                if lang == 'es_ES':
                    f.write("\tEl CT {0} no aparece en el formulario "
                            "F11.\n".format(ct))
                else:
                    f.write("\tEl CT {0} no apareix al formulari F11.\n".format(ct))
            f.write("\nTotal CTs {0}: {1}\n\n".format(context, len(cts)))
        else:
            if lang == 'es_ES':
                f.write("Los CTS del formulario {0} aparecen en el f11. "
                        "OK!\n\n".format(context))
            else:
                f.write("Els CTs del formulari {0} apareixen a l'f11. "
                        "OK!\n\n".format(context))


def comprovar_cts(ruta_f11, ruta_fitxer, context):
    cts = []
    index = 0
    cts_f11 = []
    cts_fitxer = []
    if context == 'f12':
        index = 1
    if existeix_fitxer(ruta_f11):
        if existeix_fitxer(ruta_fitxer):
            # Carreguem els CTs de l'F11
            with open(ruta_f11, 'r') as f11:
                for linia in f11.readlines():
                    linia = linia.rstrip()
                    ct = linia.split(';')[1]
                    if ct not in cts_f11:
                        cts_f11.append(ct)
            # Comprovem els CTS amb l'F11
            with open(ruta_fitxer, 'r') as fitxer:
                for linia in fitxer.readlines():
                    linia = linia.rstrip()
                    ct = linia.split(';')[index]
                    cts_fitxer.append(ct)
            cts = set(cts_fitxer) - set(cts_f11)
            cts = list(cts)
    return cts


def comprovar_repetits(ruta_fitxer, context):
    trams = []
    repetits = []
    if existeix_fitxer(ruta_fitxer):
        with open(ruta_fitxer, 'r') as fitxer:
            for linia in fitxer.readlines():
                linia = linia.rstrip()
                if context == 'f9':
                    if ';' not in linia and linia != 'END' and linia != '':
                        if linia not in trams:
                            trams.append(linia)
                        else:
                            repetits.append(linia)
                elif context == 'f10':
                    tram = linia.rsplit(';')[0]
                    if tram not in trams:
                        trams.append(tram)
                    else:
                        repetits.append(tram)
                elif context == 'f12bis':
                    posicio = linia.split(';')[2]
                    if posicio not in trams:
                        trams.append(posicio)
                    else:
                        repetits.append(posicio)
    return repetits


def check_trams_f15(ruta_f9, ruta_f15):
    trams = []
    if existeix_fitxer(ruta_f9):
        if existeix_fitxer(ruta_f15):
            trams_f9 = []
            trams_f15 = []
            # llegim tots els trams de l'F9
            with open(ruta_f9, 'r') as f9:
                for linia in f9.readlines():
                    linia = linia.rstrip()
                    if ';' not in linia and linia != 'END' and linia != '':
                        trams_f9.append(linia)
            # llegim tots els trams de l'F15
            with open(ruta_f15, 'r') as f15:
                for linia in f15.readlines():
                    linia = linia.rstrip()
                    trams_f15.append(linia.split(';')[2])
            # comparem trams de f15 si son a f9
            resta = set(trams_f15) - set(trams_f9)
            trams = [(x, 'f9', 'f15') for x in resta]
    return trams


def check_trams_f9_f10(ruta_f10, ruta_f9):
    trams = []
    if existeix_fitxer(ruta_f10):
        if existeix_fitxer(ruta_f9):
            trams_f9 = []
            trams_f10 = []
            topologia = []
            # llegim tots els trams de l'F9
            with open(ruta_f9, 'r') as f9:
                for linia in f9.readlines():
                    linia = linia.rstrip()
                    if linia == '':
                        topologia.append(trams_f9.pop())
                    elif linia and ';' not in linia and linia != 'END':
                        trams_f9.append(linia)
            # llegim tots els trams de l'F10
            with open(ruta_f10, 'r') as f10:
                for linia in f10.readlines():
                    linia = linia.rstrip()
                    if linia:
                        trams_f10.append(linia.split(';')[0])
            # comparem trams de f9 si son a f10
            resta = set(trams_f9) - set(trams_f10)
            trams = [(x, 'f10', 'f9') for x in resta]
            # comparem trams de f10 si son a f9
            resta = set(trams_f10) - set(trams_f9)
            trams += [(x, 'f9', 'f10') for x in resta]
    return trams, topologia


def mostrar_trams(trams, ruta, context, lang, topologia=None):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        repetit = ''
        if trams:
            if context == 'f9_f10':
                if lang == 'es_ES':
                    f.write("Tramos de los formularios F9 y F10:\n\n")
                else:
                    f.write("Trams dels formularis F9 i F10:\n\n")
            elif context == 'f15':
                if lang == 'es_ES':
                    f.write("Tramos del formulario F15:\n\n")
                else:
                    f.write("Trams del formulari F15:\n\n")
            elif context == 'f9r':
                repetit = 'F9'
                if lang == 'es_ES':
                    f.write("Tramos repetidos del formulario F9:\n\n")
                else:
                    f.write("Trams repetits del formulari F9:\n\n")
            elif context == 'f10r':
                repetit = 'F10'
                if lang == 'es_ES':
                    f.write("Tramos repetidos del formulario F10:\n\n")
                else:
                    f.write("Trams repetits del formulari F10:\n\n")
            for tram in trams:
                if 'r' not in context:
                    if lang == 'es_ES':
                        f.write("\tEl tramo {0} del formulario {1} no aparece "
                                "en el formulario {2}\n".format(
                            tram[0], tram[2], tram[1])
                        )
                    else:
                        f.write("\tEl tram {0} del formulari {1} no apareix "
                                "al formulari {2}\n".format(
                            tram[0], tram[2], tram[1])
                        )
                else:
                    if lang == 'es_ES':
                        f.write("\tEl tramo {0} se repite en el formulario "
                                "{1}\n".format(tram, repetit))
                    else:
                        f.write("\tEl tram {0} es repeteix al formulari "
                                "{1}\n".format(tram, repetit))
            if lang == 'es_ES':
                f.write('\nTotal tramos: {0}\n\n'.format(len(trams)))
            else:
                f.write('\nTotal trams: {0}\n\n'.format(len(trams)))
        else:
            if context == 'f9_f10':
                if lang == 'es_ES':
                    f.write("Los tramos de los formularios f9 y f10 "
                            "coinciden entre si. OK!\n\n")
                else:
                    f.write("Els trams dels formularis f9 i f10 "
                            "coincideixen entre ells. OK!\n\n")
            elif context == 'f15':
                if lang == 'es_ES':
                    f.write("Los tramos del formulario f15 coinciden con "
                            "los del formulario f9. OK!\n\n")
                else:
                    f.write("Els trams del formulari f15 coincideixen "
                            "amb els del formulari f9. OK!\n\n")
            elif context == 'f10r':
                if lang == 'es_ES':
                    f.write("No se repite ningún tramo en el formulario "
                            "f10. OK!\n\n")
                else:
                    f.write("No es repeteix cap tram al formulari f10. OK!\n\n")
            elif context == 'f9r':
                if lang == 'es_ES':
                    f.write("No se repite ningún tramo en el "
                            "formulario f9. OK!\n\n")
                else:
                    f.write("No es repeteix cap tram al formulari f9. OK!\n\n")
        if topologia:
            if lang == 'es_ES':
                f.write("Tramos del formulario F9 fuera de topología:\n\n")
            else:
                f.write("Trams del formulari F9 fora de topologia:\n\n")
            for tram in topologia:
                if lang == 'es_ES':
                    f.write("\tTramo fuera de topología: {0}\n".format(tram))
                else:
                    f.write("\tTram fora de topologia: {0}\n".format(tram))
            if lang == 'es_ES':
                f.write('\nTotal fuera topología: '
                        '{0}\n\n'.format(len(topologia)))
            else:
                f.write('\nTotal fora topologia: '
                        '{0}\n\n'.format(len(topologia)))


def mostrar_nodes(nodes, form, ruta, lang):
    fitxer_sortida = ruta+'/resultats.txt'
    with open(fitxer_sortida, 'a') as f:
        if nodes:
            if lang == 'es_ES':
                f.write(
                    "Nodos del formulario {} que no aparecen en el formulario "
                    "f10:\n\n".format(form))
            else:
                f.write(
                    "Nodes del formulari {} que no apareixen al formulari "
                    "f10:\n\n".format(form))
            for elem in nodes:
                if form == 'f1' or form == 'f2' or form == 'f6':
                    if lang == 'es_ES':
                        f.write("\tNodo: {0}, "
                                "CUPS: {1}\n".format(elem[0], elem[1]))
                    else:
                        f.write("\tNode: {0}, "
                                "CUPS: {1}\n".format(elem[0], elem[1]))
                else:
                    f.write("\t{0}\n".format(elem))
            if lang == 'es_ES':
                f.write("\nTotal nodos {0}: {1}\n\n".format(form, len(nodes)))
            else:
                f.write("\nTotal nodes {0}: {1}\n\n".format(form, len(nodes)))
        else:
            if lang == 'es_ES':
                f.write("Los nodos del formulario {0} coinciden con los del "
                        "formulario f10. OK!\n\n".format(form))
            else:
                f.write("Els nodes del formulari {0} coincideixen amb els del "
                        "formulari f10. OK!\n\n".format(form))


def check(f10_path, file_path, form):
    nodes = []
    if existeix_fitxer(f10_path):
        if existeix_fitxer(file_path):
            nodes_f10 = []
            with open(f10_path, 'r') as f10:
                node_actual = ''
                for linia_f10 in f10.readlines():
                    linia_f10 = linia_f10.rstrip()
                    if linia_f10 != '':
                        node_actual = linia_f10.split(';')[1]
                        nodes_f10.append(node_actual)
                        node_actual = linia_f10.split(';')[2]
                        nodes_f10.append(node_actual)
            # Ja tenim tots els nodes de F10, ara comprovem que cada node
            # del fitxer a comprovar és dins de F10
            with open(file_path, 'r') as f:
                for linia in f.readlines():
                    if form == 'f1' or form == 'f2' or form == 'f3' \
                            or form == 'f6' or form == 'f11' or form == 'f12' \
                            or form == 'f15' or form == 'f16' or form == 'f17':
                        node_actual = linia.split(';')[0]
                    elif form == 'f13bis':
                        node_actual = linia.split(';')[2]
                    if node_actual not in nodes_f10:
                        cups = 'formulari_sense_cups'
                        if node_actual == '':
                            node_actual = "Node buit"
                        if form == 'f1':
                            cups = linia.split(';')[8]
                        elif form == 'f2':
                            cups = linia.split(';')[3]
                        elif form == 'f6':
                            cups = linia.split(';')[4]
                        if not cups:
                            cups = "Sense CUPS"
                        if cups == 'formulari_sense_cups':
                            nodes.append(node_actual)
                        else:
                            nodes.append((node_actual, cups))
    return nodes


def existeix_fitxer(ruta_fitxer):
    if os.path.exists(ruta_fitxer):
        return True
    else:
        print "El fitxer {0} no existeix.".format(ruta_fitxer)
        return False


if __name__ == '__main__':
    node_check()



# -*- coding: utf-8 -*-
import os

import click
from libcnmc.utils import N_PROC
from libcnmc.core.backend import OOOPFactory
import tempfile
from datetime import datetime


@click.group()
def cnmc_4131():
    pass


# CSV POS
def res_pos2(proc1, proc2, **kwargs):
    """

    :param proc1: generation proces 1
    :param proc2: generation proces 1
    :param kwargs:
    :return:
    """

    O = OOOPFactory(dbname=kwargs["database"], user=kwargs["user"],
                    pwd=kwargs["password"], port=kwargs["port"],
                    uri=kwargs["server"])
    output = kwargs["output"]
    temp_fd = tempfile.NamedTemporaryFile()

    tmp_out1 = temp_fd.name
    temp_fd.close()
    temp_fd = tempfile.NamedTemporaryFile()
    tmp_out2 = temp_fd.name
    temp_fd.close()

    proc = proc1(
        quiet=kwargs["quiet"],
        interactive=kwargs["interactive"],
        output=tmp_out1,
        connection=O,
        num_proc=kwargs["num_proc"],
        codi_r1=kwargs["codi_r1"],
        year=kwargs["year"],
        embarrats=kwargs["embarrats"],
        compare_field=kwargs["compare_field"]
    )
    proc.calc()

    proc_2 = proc2(
        quiet=kwargs["quiet"],
        interactive=kwargs["interactive"],
        output=tmp_out2,
        connection=O,
        num_proc=kwargs["num_proc"],
        codi_r1=kwargs["codi_r1"],
        year=kwargs["year"],
        embarrats=kwargs["embarrats"],
        compare_field=kwargs["compare_field"]
    )
    proc_2.calc()

    final_out = ""
    with open(tmp_out1, 'r') as fd1:
        final_out += fd1.read()

    with open(tmp_out2, 'r') as fd2:
        final_out += fd2.read()

    with open(output, 'w') as fd_out:
        fd_out.write(final_out)

    os.unlink(tmp_out1)
    os.unlink(tmp_out2)


# CSV LAT
def res_lat(LAT, **kwargs):
    """
    Generates the CSV file for LAT process

    :param LAT: process to generate
    :type LAT: MultiprocessBased
    :param kwargs: Parameters to generate the file, passed to the process
    :type kwargs: dict(str,str)
    :return: None
    :rtype: None
    """

    O = OOOPFactory(dbname=kwargs['database'], user=kwargs['user'],
                    pwd=kwargs['password'], port=kwargs['port'],
                    uri=kwargs['server'])
    proc = LAT(
        quiet=kwargs['quiet'],
        interactive=kwargs['interactive'],
        output=kwargs['output'],
        connection=O,
        num_proc=kwargs['num_proc'],
        codi_r1=kwargs['codi_r1'],
        year=kwargs['year'],
        embarrats=kwargs['embarrats'],
        compare_field=kwargs["compare_field"]
    )
    proc.calc()


@cnmc_4131.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--embarrats/--no-embarrats', default=False,
              help="Afegir embarrats")
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4131_lat(**kwargs):
    """
    Click entry to gnerate the LAT of 4131

    :param kwargs: Params to pas to the process
    :type kwargs: dict(str, str) 
    :return: None
    :rtype: None 
    """
    from libcnmc.res_4131 import LAT, LAT_2015
    if kwargs['year'] == 2015:
        kwargs["compare_field"] = "4771_entregada"
        res_lat(LAT_2015, **kwargs)
    else:
        last_year = datetime.now().year - 1
        kwargs["compare_field"] = "4131_entregada_{}".format(last_year)
        res_lat(LAT, **kwargs)


@cnmc_4131.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--embarrats/--no-embarrats', default=False,
              help="Afegir embarrats")
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4131_lbt(**kwargs):
    """
    Click entry to generate the LBT of 4131

    :param kwargs: Params to pas to the process
    :type kwargs: dict(str, str) 
    :return: None
    :rtype: None 
    """
    from libcnmc.res_4131 import LBT, LBT_2015
    if kwargs['year'] == 2015:
        kwargs["compare_field"] = "4771_entregada"
        res_lat(LBT_2015, **kwargs)
    else:
        last_year = datetime.now().year - 1
        kwargs["compare_field"] = "4131_entregada_{}".format(last_year)
        res_lat(LBT, **kwargs)


@cnmc_4131.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--embarrats/--no-embarrats', default=False,
              help="Afegir embarrats")
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4131_cts(**kwargs):
    """
    Click entry to generate the CTs of 4131

    :param kwargs: Params to pas to the process
    :type kwargs: dict(str, str) 
    :return: None
    :rtype: None 
    """

    from libcnmc.res_4131 import CTS, CTS_2015
    if kwargs['year'] == 2015:
        kwargs["compare_field"] = "4771_entregada"
        res_lat(CTS_2015, **kwargs)
    else:
        kwargs["compare_field"] = "4131_entregada_{}".format(kwargs["year"])
        res_lat(CTS, **kwargs)


@cnmc_4131.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--embarrats/--no-embarrats', default=False,
              help="Afegir embarrats")
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4131_sub(**kwargs):
    """
    Click entry to generate the SUB fil of 4131

    :param kwargs: Params to pas to the process
    :type kwargs: dict(str, str) 
    :return: None
    :rtype: None 
    """

    from libcnmc.res_4131 import SUB, SUB_2015
    if kwargs['year'] == 2015:
        kwargs["compare_field"] = "4771_entregada"
        res_lat(SUB_2015, **kwargs)
    else:
        kwargs["compare_field"] = "4131_entregada_{}".format(kwargs["year"])
        res_lat(SUB, **kwargs)


@cnmc_4131.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--embarrats/--no-embarrats', default=False,
              help="Afegir embarrats")
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4131_pos(**kwargs):
    """
    Click entry to generate the POS file of 4131

    :param kwargs: Params to pas to the process
    :type kwargs: dict(str, str) 
    :return: None
    :rtype: None 
    """

    from libcnmc.res_4131 import POS, POS_2015, POS_INT
    if kwargs['year'] == 2015:
        kwargs["compare_field"] = "4771_entregada"
        res_lat(POS_2015, **kwargs)
    else:
        kwargs["compare_field"] = "4131_entregada_{}".format(kwargs["year"])
        res_pos2(POS, POS_INT, **kwargs)


@cnmc_4131.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--embarrats/--no-embarrats', default=False,
              help="Afegir embarrats")
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4131_maq(**kwargs):
    """
    Click entry to generate the MAQ file of 4131

    :param kwargs: Params to pas to the process
    :type kwargs: dict(str, str) 
    :return: None
    :rtype: None 
    """

    from libcnmc.res_4131 import MAQ, MAQ_2015
    if kwargs["year"] == 2015:
        kwargs["compare_field"] = "4771_entregada"
        res_lat(MAQ_2015, **kwargs)
    else:
        kwargs["compare_field"] = "4131_entregada_{}".format(kwargs["year"])
        res_lat(MAQ, **kwargs)


@cnmc_4131.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--embarrats/--no-embarrats', default=False,
              help="Afegir embarrats")
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4131_des(**kwargs):
    """
    Click entry to generate the DES file

    :param kwargs: Params to pas to the process
    :type kwargs: dict(str, str) 
    :return: None
    :rtype: None 
    """

    from libcnmc.res_4131 import DES, DES_2015
    if kwargs['year'] == 2015:
        kwargs["compare_field"] = "4771_entregada"
        res_lat(DES_2015, **kwargs)
    else:
        kwargs["compare_field"] = "4131_entregada_{}".format(kwargs["year"])
        res_lat(DES, **kwargs)


@cnmc_4131.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--embarrats/--no-embarrats', default=False,
              help="Afegir embarrats")
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4131_fia(**kwargs):
    """
    Click entry to generate the fiabilitat file of 4131

    :param kwargs: Parameters to generate the file
    :type kwargs: dict(str,str)
    :return: None
    :rtype: None
    """
    from libcnmc.res_4131 import FIA, FIA_2015
    if kwargs['year'] == 2015:
        kwargs["compare_field"] = "4771_entregada"
        res_lat(FIA_2015, **kwargs)
    else:
        kwargs["compare_field"] = "4131_entregada_{}".format(kwargs["year"])
        res_lat(FIA, **kwargs)


@cnmc_4131.command()
@click.option('-q', '--quiet', default=False,
              help="No mostrar missatges de status per stderr")
@click.option('--interactive/--no-interactive', default=True,
              help="Deshabilitar el mode interactiu")
@click.option('-o', '--output', help="Fitxer de sortida")
@click.option('-c', '--codi-r1', help='Codi R1 de la distribuidora')
@click.option('-y', '--year', default=(datetime.now().year - 1),
              help=u"Any per càlculs")
@click.option('-s', '--server', default='http://localhost',
              help=u'Adreça servidor ERP')
@click.option('-p', '--port', default=8069, help='Port servidor ERP',
              type=click.INT)
@click.option('-u', '--user', default='admin', help='Usuari servidor ERP')
@click.option('-w', '--password', default='admin',
              help='Contrasenya usuari ERP')
@click.option('-d', '--database', help='Nom de la base de dades')
@click.option('--embarrats/--no-embarrats', default=False,
              help="Afegir embarrats")
@click.option('--num-proc', default=N_PROC, type=click.INT)
def res_4131_con(**kwargs):
    """
    Click entry to generate the file of condensaodrs of 4131

    :param kwargs: Params to generate
    :type kwargs: dict(str, str)
    :return: None
    :rtype: None
    """

    from libcnmc.res_4131 import CON
    if kwargs['year'] == 2015:
        kwargs["compare_field"] = "4771_entregada"
        res_lat(CON, **kwargs)
    else:
        kwargs["compare_field"] = "4131_entregada_{}".format(kwargs["year"])
        res_lat(CON, **kwargs)


# -*- coding: utf-8 -*-
from datetime import datetime
import traceback
from libcnmc.utils import format_f, tallar_text
from libcnmc.core import StopMultiprocessBased


class F10AT(StopMultiprocessBased):
    def __init__(self, **kwargs):
        super(F10AT, self).__init__(**kwargs)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'F10AT - LAT'
        self.base_object = 'AT'
        self.layer = 'LBT\_%'
        id_res_like = self.connection.ResConfig.search(
            [('name', '=', 'giscegis_btlike_layer')])
        if id_res_like:
            self.layer = self.connection.ResConfig.read(
                id_res_like, ['value'])[0]['value']
        o = self.connection
        trafos_fields = o.GiscedataTransformadorTrafo.fields_get().keys()
        if 'node_id' in trafos_fields:
            ids_red = o.GiscedataTransformadorTrafo.search(
                [('reductor', '=', True)]
            )
            data_nodes = o.GiscedataTransformadorTrafo.read(
                ids_red, ["node_id"]
            )
            self.nodes_red = [
                nod["node_id"][1] for nod in data_nodes if nod["node_id"]
            ]
        else:
            ids_red = o.GiscegisBlocsTransformadorsReductors.search([])
            data_nodes = o.GiscegisBlocsTransformadorsReductors.read(
                ids_red, ["node"]
            )
            self.nodes_red = [
                nod["node"][1] for nod in data_nodes if nod["node"]
            ]

    def get_sequence(self):
        search_params = [('name', '!=', '1')]
        lines_id = self.connection.GiscedataAtLinia.search(search_params)
        search_params = [('name', '=', '1')]
        fict_line_id = self.connection.GiscedataAtLinia.search(
            search_params, 0, 0, False, {'active_test': False})
        return lines_id + fict_line_id

    def get_provincia(self, id_prov):
        o = self.connection
        res = ''
        ine = o.ResCountryState.read(id_prov, ['code'])['code']
        if ine:
            res = ine
        return res

    def consumer(self):
        o = self.connection
        fields_to_read = [
            'name', 'cini', 'circuits', 'longitud_cad', 'linia', 'origen',
            'final', 'coeficient', 'cable', 'tensio_max_disseny_id', 'id_regulatori'
        ]
        data_pm_limit = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        static_search_params = [
            ('cini', '!=', '0000000'),
            '|',
            ('data_pm', '=', False),
            ('data_pm', '<', data_pm_limit),
            '|',
            ('data_baixa', '>', data_baixa),
            ('data_baixa', '=', False),
        ]
        # Revisem que si està de baixa ha de tenir la data informada.
        static_search_params += ['|',
                                 '&', ('active', '=', False),
                                      ('data_baixa', '!=', False),
                                 ('active', '=', True)]
        while True:
            try:
                # generar linies
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                linia = o.GiscedataAtLinia.read(
                    item, ['trams', 'tensio', 'municipi', 'propietari',
                           'provincia']
                )

                o_prop = linia['propietari'] and '1' or '0'
                search_params = [('id', 'in', linia['trams'])]
                search_params += static_search_params
                ids = o.GiscedataAtTram.search(
                    search_params, 0, 0, False, {'active_test': False})
                for at in o.GiscedataAtTram.read(ids, fields_to_read):
                    # Coeficient per ajustar longituds de trams
                    coeficient = at['coeficient'] or 1.0
                    # Comprovar el tipus del cable
                    fields_to_read_cable = [
                        'tipus', 'resistencia', 'reactancia',
                        'intensitat_admisible'
                    ]
                    cable = o.GiscedataAtCables.read(at['cable'][0],
                                                     fields_to_read_cable)
                    o_tipus = o.GiscedataAtTipuscable.read(cable['tipus'][0],
                                                           ['codi'])['codi']
                    # Si el tram tram es embarrat amb una longitud > 100
                    # no l'afegim
                    if o_tipus == 'E' and at['longitud_cad'] > 100:
                        continue
                    if o_tipus == 'E':
                        o_tipus = 'D'
                    # Agafem la tensió
                    if at.get('tensio_max_disseny_id', False):
                        nivell_tensio_id = at['tensio_max_disseny_id'][0]
                        o_nivell_tensio = o.GiscedataTensionsTensio.read(nivell_tensio_id, ["tensio"])["tensio"]
                    else:
                        o_nivell_tensio = linia["tensio"]
                    o_nivell_tensio = format_f(
                        float(o_nivell_tensio) / 1000.0, 3)

                    # Si hi ha 'id_regulatori' el posem
                    if at.get('id_regulatori', False):
                        o_tram = at['id_regulatori']
                    else:
                        o_tram = 'A%s' % at['name']

                    if 'edge_id' in o.GiscedataAtTram.fields_get().keys():
                        at_edge = o.GiscedataAtTram.read(
                            at['id'], ['edge_id']
                        )['edge_id']
                        if not at_edge:
                            res = o.GiscegisEdge.search(
                                [
                                    ('id_linktemplate', '=', at['name']),
                                    ('layer', 'not ilike', self.layer),
                                    ('layer', 'not ilike', 'EMBARRA%BT%')
                                ]
                            )
                            if not res or len(res) > 1:
                                edge = {
                                    'start_node': (0, '%s_0' % at['name']),
                                    'end_node': (0, '%s_1' % at['name'])
                                }
                            else:
                                edge = o.GiscegisEdge.read(
                                    res[0], ['start_node', 'end_node']
                                )
                        else:
                            edge = o.GiscegisEdge.read(
                                at_edge[0], ['start_node', 'end_node']
                            )
                    else:
                        res = o.GiscegisEdge.search(
                            [
                                ('id_linktemplate', '=', at['name']),
                                ('layer', 'not ilike', self.layer),
                                ('layer', 'not ilike', 'EMBARRA%BT%')
                            ]
                        )
                        if not res or len(res) > 1:
                            edge = {
                                'start_node': (0, '%s_0' % at['name']),
                                'end_node': (0, '%s_1' % at['name'])
                            }
                        else:
                            edge = o.GiscegisEdge.read(
                                res[0], ['start_node', 'end_node']
                            )
                    o_node_inicial = tallar_text(edge['start_node'][1], 20)
                    o_node_inicial = o_node_inicial.replace('*', '')
                    if o_node_inicial in self.nodes_red:
                        o_node_inicial = "{}-{}".format(
                            o_node_inicial, o_nivell_tensio
                        )
                    o_node_final = tallar_text(edge['end_node'][1], 20)
                    o_node_final = o_node_final.replace('*', '')
                    if o_node_final in self.nodes_red:
                        o_node_final = "{}-{}".format(
                            o_node_final, o_nivell_tensio
                        )
                    o_cini = at['cini']
                    o_provincia = ''
                    if linia['provincia']:
                        o_provincia = self.get_provincia(linia['provincia'][0])
                    o_longitud = format_f(
                            round(float(at['longitud_cad']) *
                                  coeficient / 1000.0, 3) or 0.001, decimals=3)
                    o_num_circuits = at['circuits']
                    o_r = format_f(
                        cable['resistencia'] * (float(at['longitud_cad']) *
                                                coeficient / 1000.0) or 0.0,
                        decimals=6)
                    o_x = format_f(
                        cable['reactancia'] * (float(at['longitud_cad']) *
                                               coeficient / 1000.0) or 0.0,
                        decimals=6)
                    o_int_max = format_f(
                        cable['intensitat_admisible'] or 0.0, decimals=3)
                    o_op_habitual = 1  # Tots son actius
                    o_cod_dis = 'R1-%s' % self.codi_r1[-3:]
                    o_any = self.year

                    self.output_q.put([
                        o_tram,             # TRAMO
                        o_node_inicial,     # NUDO INICIAL
                        o_node_final,       # NUDO FINAL
                        o_cini,             # CINI
                        o_provincia,        # PROVINCIA
                        o_nivell_tensio,    # NIVEL TENSION
                        o_longitud,         # LONGITUD
                        o_num_circuits,     # NUMERO CIRCUITOS
                        o_tipus,            # TIPO
                        o_r,                # RESISTENCIA
                        o_x,                # REACTANCIA
                        o_int_max,          # INTENSIDAD MAXIMA
                        o_op_habitual,      # ESTADO OPERACION HABITUAL
                        o_cod_dis,          # CODIGO DISTRIBUIDORA
                        o_prop,             # PROPIEDAD
                        o_any               # AÑO INFORMACION
                    ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



from LAT import LAT, LAT_2015
from LBT import LBT, LBT_2015
from SUB import SUB, SUB_2015
from POS import POS, POS_2015, POS_INT
from MAQ import MAQ, MAQ_2015
from DES import DES, DES_2015
from FIA import FIA, FIA_2015
from CTS import CTS, CTS_2015
from CON import CON



# -*- coding: utf-8 -*-
from datetime import datetime
import traceback
from libcnmc.utils import format_f, tallar_text
from libcnmc.core import StopMultiprocessBased


class F10BT(StopMultiprocessBased):
    def __init__(self, **kwargs):
        super(F10BT, self).__init__(**kwargs)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'F10BT - LBT'
        self.base_object = 'BT'
        self.layer = 'LBT\_%'
        id_res_like = self.connection.ResConfig.search(
            [('name', '=', 'giscegis_btlike_layer')])
        if id_res_like:
            self.layer = self.connection.ResConfig.read(
                id_res_like, ['value'])[0]['value']

    def get_sequence(self):
        search_params = [
            ('cable.tipus.codi', 'in', ['T', 'D', 'S', 'E', 'I'])
        ]
        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        search_params += ['|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                               ('data_baixa', '=', False),
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataBtElement.search(
            search_params, 0, 0, False, {'active_test': False})

    def get_tipus_cable(self, id_cable, id_tipus_linia):
        o = self.connection
        cable = o.GiscedataBtTipuscable.read(id_cable, ['codi'])
        tipus_linia = o.GiscedataBtTipuslinia.read(
            id_tipus_linia, ['name'])['name'][0]
        res = 'I'
        if tipus_linia == 'S':
            # Subterrània
            res = 'S'
        elif tipus_linia == 'A':
            # Aèria
            if cable:
                codi = cable['codi']
                if codi in ['S', 'T', 'I']:
                    res = 'T'
                elif codi == 'D':
                    res = 'D'
                elif codi == 'E':
                    res = 'D'
        return res

    def get_provincia(self, id_mun):
        o = self.connection
        res = ''
        prov = o.ResMunicipi.read(id_mun, ['ine'])['ine']
        if prov:
            res = prov[0:2]
        return res

    def consumer(self):
        o = self.connection
        fields_to_read = [
            'name', 'propietari', 'coeficient', 'cable', 'voltatge', 'cini',
            'longitud_cad', 'municipi', 'longitud_cad', 'tipus_linia', 'id_regulatori'
        ]
        while True:
            try:
                # generar linies
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                linia = o.GiscedataBtElement.read(item, fields_to_read)

                o_prop = linia['propietari'] and '1' or '0'
                # Coeficient per ajustar longituds de trams
                coeficient = linia['coeficient'] or 1.0
                # Comprovar el tipus del cable
                fields_to_read_cable = [
                    'tipus', 'reactancia', 'resistencia', 'intensitat_admisible'
                ]
                # Agafem el cable de la linia
                cable = o.GiscedataBtCables.read(linia['cable'][0],
                                                 fields_to_read_cable)
                #Agafem la tensió
                try:
                    o_nivell_tensio = format_f(
                        (int(linia['voltatge']) / 1000.0), 3
                    )
                except:
                    o_nivell_tensio = 0.0

                # Si hi ha 'id_regulatori' el posem
                if linia.get('id_regulatori', False):
                    o_tram = linia['id_regulatori']
                else:
                    o_tram = 'B%s' % linia['name']

                if 'edge_id' in o.GiscedataBtElement.fields_get().keys():
                    bt_edge = o.GiscedataBtElement.read(
                        linia['id'], ['edge_id']
                    )['edge_id']
                    if not bt_edge:
                        res = o.GiscegisEdge.search(
                            [('id_linktemplate', '=', linia['name']),
                             '|',
                             ('layer', 'ilike', self.layer),
                             ('layer', 'ilike', 'EMBARRA%BT%')
                             ])
                        if not res or len(res) > 1:
                            edge = {'start_node': (0, '%s_0' % linia['name']),
                                    'end_node': (0, '%s_1' % linia['name'])}
                        else:
                            edge = o.GiscegisEdge.read(
                                res[0], ['start_node', 'end_node']
                            )
                    else:
                        edge = o.GiscegisEdge.read(
                            bt_edge[0], ['start_node', 'end_node']
                        )
                else:
                    res = o.GiscegisEdge.search(
                        [('id_linktemplate', '=', linia['name']),
                         '|',
                         ('layer', 'ilike', self.layer),
                         ('layer', 'ilike', 'EMBARRA%BT%')
                         ])
                    if not res or len(res) > 1:
                        edge = {'start_node': (0, '%s_0' % linia['name']),
                                'end_node': (0, '%s_1' % linia['name'])}
                    else:
                        edge = o.GiscegisEdge.read(res[0], ['start_node',
                                                            'end_node'])
                o_node_inicial = tallar_text(edge['start_node'][1], 20)
                o_node_inicial = o_node_inicial.replace('*', '')
                o_node_final = tallar_text(edge['end_node'][1], 20)
                o_node_final = o_node_final.replace('*', '')
                o_cini = linia['cini']
                o_provincia = ''
                if linia['municipi']:
                    o_provincia = self.get_provincia(linia['municipi'][0])
                o_longitud = format_f(
                        round(float(linia['longitud_cad']) *
                              coeficient / 1000.0, 3) or 0.001, decimals=3)
                o_num_circuits = 1  # a BT suposarem que sempre hi ha 1
                o_tipus = self.get_tipus_cable(
                    cable['tipus'][0], linia['tipus_linia'][0]
                )
                o_r = format_f(
                    cable['resistencia'] * (float(linia['longitud_cad']) * coeficient / 1000.0) or 0.0, 6)
                o_x = format_f(
                    cable['reactancia'] * (float(linia['longitud_cad']) * coeficient / 1000.0) or 0.0, 6)
                o_int_max = format_f(cable['intensitat_admisible'], 3)
                o_op_habitual = 1  # Tots son actius
                o_cod_dis = 'R1-%s' % self.codi_r1[-3:]
                o_any = self.year

                if cable['tipus'][1] in ['EMBARRADO', 'EMBARRAT']:
                    o_prop = 1

                self.output_q.put([
                    o_tram,             # TRAMO
                    o_node_inicial,     # NUDO INICIAL
                    o_node_final,       # NUDO FINAL
                    o_cini,             # CINI
                    o_provincia,        # PROVINCIA
                    o_nivell_tensio,    # NIVEL TENSION
                    o_longitud,         # LONGITUD
                    o_num_circuits,     # NUMERO CIRCUITOS
                    o_tipus,            # TIPO
                    o_r,                # RESISTENCIA
                    o_x,                # REACTANCIA
                    o_int_max,          # INTENSIDAD MAXIMA
                    o_op_habitual,      # ESTADO OPERACION HABITUAL
                    o_cod_dis,          # CODIGO DISTRIBUIDORA
                    o_prop,             # PROPIEDAD
                    o_any               # AÑO INFORMACION
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



# -*- coding: utf-8 -*-
from dateutil.parser import parse
from datetime import datetime

from libcnmc.res_4603.create_celles import CreateCelles

class CreateCelles4_2015(CreateCelles):
    def __init__(self, **kwargs):
        super(CreateCelles4_2015, self).__init__(**kwargs)
        self.header += ['cnmc_tipo_instalacion', 'data_pm', 'circuit']
        self.fields_read_ct += ['data_pm']
        self.fields_read_at_tram += ['data_pm']

    def build_vals(self, values):
        vals = super(CreateCelles4_2015, self).build_vals(values)
        o = self.connection
        ct_name = False
        suport_name = False
        for val in zip(self.header, values):
            if val[0] == 'installacio':
                model, name = val[1].split(',')
                model = model.lower()
                if 'ct' in model:
                    ct_name = name
                elif 'suport' in model:
                    suport_name = name
                else:
                    raise Exception("No element")
            elif val[0] == 'data_pm':
                vals['bloquejar_pm'] = True
                if val[1] == 'auto':
                    if ct_name:
                        tipus = 'ct'
                        name = ct_name
                    if suport_name:
                        tipus = 'suport'
                        name = suport_name
                    vals[val[0]] = self.get_value(
                        tipus, name, 'data_pm')
                else:
                    date = parse(val[1])
                    vals[val[0]] = date.strftime('%Y-%m-%d')
            elif val[0] == 'circuit':
                search_param = [('name', '=', val[1])]
                value = o.GiscedataCtsSubestacionsPosicio.search(
                    search_param, 0, 0, False, {'active_test': False})[0]
                vals['posicio_id'] = value
        return vals




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Posicions
"""
from datetime import datetime
import traceback
import sys

from libcnmc.core import MultiprocessBased
from libcnmc.utils import get_id_municipi_from_company, format_f

QUIET = False


class POS(MultiprocessBased):
    def __init__(self, **kwargs):
        super(POS, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'Línies POS'
        self.report_name = 'CNMC INVENTARI POS'

    def get_sequence(self):
        search_params = [('interruptor', '=', '2')]
        data_pm = '%s-01-01' % (self.year + 1)
        data_baixa = '%s-12-31' % self.year
        search_params += [('propietari', '=', True),
                          '|', ('data_pm', '=', False),
                               ('data_pm', '<', data_pm),
                          '|', ('data_baixa', '>', data_baixa),
                               ('data_baixa', '=', False),
                          ]
        # Revisem que si està de baixa ha de tenir la data informada.
        search_params += ['|',
                          '&', ('active', '=', False),
                               ('data_baixa', '!=', False),
                          ('active', '=', True)]
        return self.connection.GiscedataCtsSubestacionsPosicio.search(
            search_params, 0, 0, False, {'active_test': False})

    def get_description(self, pos_id):
        o = self.connection
        pos = o.GiscedataCtsSubestacionsPosicio.read(pos_id, ['subestacio_id'])
        descripcio = ''
        if pos:
            sub_id = pos['subestacio_id'][0]
            sub = o.GiscedataCtsSubestacions.read(sub_id, ['descripcio'])
            if sub:
                descripcio = sub['descripcio']
        return descripcio

    def get_denom(self, sub_id):
        o = self.connection
        res = ''
        ct_id = o.GiscedataCtsSubestacions.read(sub_id, ['ct_id'])['ct_id'][0]
        if ct_id:
            denom = o.GiscedataCts.read(ct_id, ['descripcio'])['descripcio']
            if denom:
                res = denom
        return res

    def consumer(self):
        O = self.connection
        fields_to_read = ['name', 'cini', 'data_pm', 'subestacio_id',
                          'cnmc_tipo_instalacion', 'perc_financament', 'tensio']
        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)
                sub = O.GiscedataCtsSubestacionsPosicio.read(
                    item, fields_to_read)
                if not sub:
                    txt = ('**** ERROR: El ct %s (id:%s) no està a '
                           'giscedata_cts_subestacions_posicio.\n' %
                           (sub['name'], sub['id']))
                    if not QUIET:
                        sys.stderr.write(txt)
                        sys.stderr.flush()

                    raise Exception(txt)
                o_sub = sub['name']
                # Calculem any posada en marxa
                data_pm = sub['data_pm']
                if data_pm:
                    data_pm = datetime.strptime(str(data_pm), '%Y-%m-%d')
                    data_pm = data_pm.strftime('%d/%m/%Y')

                #Codi tipus de instalació
                codi = sub['cnmc_tipo_instalacion']

                comunitat = ''

                #tensio
                ten = O.GiscedataTensionsTensio.read(sub['tensio'][0],
                                                     ['tensio'])
                tensio = (ten['tensio'] / 1000.0) or 0.0

                cts = O.GiscedataCtsSubestacions.read(sub['subestacio_id'][0],
                                                      ['id_municipi'])

                denominacio = self.get_denom(sub['subestacio_id'][0])

                if cts['id_municipi']:
                    id_municipi = cts['id_municipi'][0]
                else:
                    id_municipi = get_id_municipi_from_company(O)

                if id_municipi:
                    #funció per trobar la ccaa desde el municipi
                    fun_ccaa = O.ResComunitat_autonoma.get_ccaa_from_municipi
                    id_comunitat = fun_ccaa(id_municipi)
                    comunitat_vals = O.ResComunitat_autonoma.read(
                        id_comunitat[0], ['codi'])
                    if comunitat_vals:
                        comunitat = comunitat_vals['codi']
                    # o_sub = self.get_description(sub['subestacio_id'][0])

                output = [
                    o_sub,
                    sub['cini'] or '',
                    denominacio,
                    codi,
                    comunitat,
                    format_f(tensio),
                    format_f(round(100 - int(sub['perc_financament']))),
                    data_pm or '',
                ]

                self.output_q.put(output)
            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



# -*- coding: utf-8 -*-
from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import format_f
from datetime import datetime
import traceback


class FC4(StopMultiprocessBased):

    def __init__(self, **kwargs):
        super(FC4, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.report_name = ('Formulario C4: Detalle de inversión, '
                            'gastos e ingresos en atención de la prestación de '
                            'servicios por la distribuidora que tienen '
                            'establecido un precio regulado. '
                            'Derechos de extensión')
        self.base_object = 'Operaciones partes vinculadas'

    def get_sequence(self):
        c4_ids = self.connection.model('cir8.2021.c4').search(
            [('year', '=', self.year)]
        )
        return c4_ids

    def consumer(self):
        O = self.connection
        fields_to_read = [
            'cia', 'actuaciones', 'inversion',
            'gasto', 'ingreso', 'tension', 'potencia'
        ]
        while True:
            try:
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)

                c4 = O.model('cir8.2021.c4').read(item, fields_to_read)
                self.output_q.put([
                    c4.get('cia', ''),
                    c4.get('actuaciones', 0),
                    format_f(c4.get('inversion', 0.0), 2),
                    format_f(c4.get('gasto', 0.0), 2),
                    format_f(c4.get('ingreso', 0.0), 2),
                    format_f(c4.get('tension', 0.0), 3),
                    format_f(c4.get('potencia', 0.0), 3),
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC AT
"""
import traceback

from libcnmc.res_4667.utils import get_resum_any_id
from libcnmc.utils import get_name_ti, get_codigo_ccaa, format_f
from libcnmc.core import MultiprocessBased


class DES(MultiprocessBased):
    """
    Class to generate F6 of 4667
    """

    def __init__(self, **kwargs):
        """
        Class constructor
        
        :param kwargs:
        :type kwargs: dict
        """

        self.year = kwargs.pop("year")
        super(DES, self).__init__(**kwargs)

    def get_sequence(self):
        """
        Generates the sequence of ids to make the report

        :return: List of ids
        :rtype: list
        """

        id_resum = get_resum_any_id(self.connection, self.year)
        search_des = [("resums_inversio", "in", id_resum)]

        return self.connection.GiscedataCnmcDespatx.search(search_des)

    def consumer(self):
        """
        Generates the line of the file
        :return: Line 
        :rtype: str
        """

        O = self.connection
        fields_to_read = [
            "codi", "finalitat", "id_instalacio", "cini", "codigo_ccaa",
            "any_apm", "vol_total_inv", "ajudes", "vpi_retri", "estado",
            "actuacio_elegible_prtr"
        ]

        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                des = O.GiscedataCnmcDespatx.read(item, fields_to_read)
                output = [
                    des["codi"][1],
                    des["finalitat"],
                    des["id_instalacio"],
                    des["cini"],
                    get_codigo_ccaa(O, des["codigo_ccaa"][0]),
                    des["any_apm"],
                    format_f(des["vol_total_inv"], 2) or "0.00",
                    format_f(des["ajudes"], 2) or "0.00",
                    format_f(des["vpi_retri"], 2) or "0.00",
                    des["estado"],
                    des["actuacio_elegible_prtr"] or '',
                ]
                self.output_q.put(output)

            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



from f1_4667 import F1Res4667
from f2_4667 import F2Res4667
from f3_4667 import F3Res4667
from f4_4667 import F4Res4667
from f5_4667 import F5Res4667
from f6_4667 import F6Res4667
from f7_4667 import F7Res4667
from f8_4667 import F8Res4667
from macro_4667 import MacroRes4667
from resumen_4667 import ResumenRes4667
from resumen_ccaa_4667 import ResumenCCAARes4667
from pro_4667 import ProyectoRes4667



from f1_4771 import F1Res4771
from f2_4771 import F2Res4771
from f3_4771 import F3Res4771
from f4_4771 import F4Res4771
from f5_4771 import F5Res4771
from f6_4771 import F6Res4771
from f7_4771 import F7Res4771
from f8_4771 import F8Res4771



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import traceback
from os import environ

from libcnmc.utils import format_f_6181, get_codi_actuacio, format_ccaa_code, \
    convert_spanish_date
from libcnmc.core import MultiprocessBased


class DES(MultiprocessBased):
    """
    Class to generate installation report for DES (6)
    """

    def __init__(self, **kwargs):
        """
        Class constructor
        
        :param kwargs:
        :type kwargs: dict
        """

        self.year = kwargs.pop("year")
        super(DES, self).__init__(**kwargs)
        self.include_obres = False
        if kwargs.get("include_obra", False):
            self.include_obres = True
        if kwargs.get("include_header", False):
            self.file_header = self.get_header()

    def get_header(self):
        header = [
            'IDENTIFICADOR',
            'CINI',
            'FINANCIADO',
            'CODIGO_CCAA',
            'FECHA_APS',
            'FECHA_BAJA',
            'CAUSA_BAJA',
            'IM_INGENIERIA',
            'IM_MATERIALES',
            'IM_OBRACIVIL',
            'IM_TRABAJOS',
            'SUBVENCIONES_EUROPEAS',
            'SUBVENCIONES_NACIONALES',
            'VALOR_AUDITADO',
            'VALOR_CONTABLE',
            'CUENTA_CONTABLE',
            'MOTIVACION',
        ]
        if self.include_obres:
            header.insert(0, 'IDENTIFICADOR_OBRA')
        return header

    def get_sequence(self):
        """
        Generates the sequence of ids to make the report

        :return: List of ids
        :rtype: list
        """

        installations_ids = self.connection.GiscedataProjecteObra.get_audit_installations_by_year(
            [], self.year, [6]
        )

        return installations_ids[6]

    def consumer(self):
        """
        Generates the line of the file
        :return: Line 
        :rtype: str
        """

        O = self.connection
        fields_to_read = [
            'name',
            'cini',
            'financiado',
            'codigo_ccaa',
            'fecha_aps',
            'fecha_baja',
            'causa_baja',
            'im_ingenieria',
            'im_materiales',
            'im_obracivil',
            'im_trabajos',
            'subvenciones_europeas',
            'subvenciones_nacionales',
            'valor_auditado',
            'valor_contabilidad',
            'cuenta_contable',
            'obra_id',
            'motivacion',
        ]

        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)

                linia = O.GiscedataProjecteObraTiDespatx.read([item], fields_to_read)[0]

                fecha_aps = convert_spanish_date(
                    linia['fecha_aps'] if not linia['fecha_baja'] else ''
                )
                # Si la data APS es igual a l'any de la generació del fitxer,
                # la data APS sortirà en blanc
                fecha_aps = '' if fecha_aps and int(fecha_aps.split('/')[2]) != self.year \
                    else fecha_aps
                output = [
                    linia['name'],
                    linia['cini'],
                    format_f_6181(linia['financiado'], float_type='decimal'),
                    format_ccaa_code(linia['codigo_ccaa']),
                    fecha_aps,
                    convert_spanish_date(linia['fecha_baja']),
                    linia['causa_baja'],
                    format_f_6181(linia['im_ingenieria'] or 0.0, float_type='euro'),
                    format_f_6181(linia['im_materiales'] or 0.0, float_type='euro'),
                    format_f_6181(linia['im_obracivil'] or 0.0, float_type='euro'),
                    format_f_6181(linia['im_trabajos'] or 0.0, float_type='euro'),
                    format_f_6181(linia['subvenciones_europeas'] or 0.0, float_type='euro'),
                    format_f_6181(linia['subvenciones_nacionales'] or 0.0, float_type='euro'),
                    format_f_6181(linia['valor_auditado'] or 0.0, float_type='euro'),
                    format_f_6181(linia['valor_contabilidad'] or 0.0, float_type='euro'),
                    linia['cuenta_contable'],
                    (
                        get_codi_actuacio(O, linia['motivacion'] and linia['motivacion'][0])
                        if not linia['fecha_baja'] else ''
                    ),
                ]
                if self.include_obres:
                    output.insert(0, linia['obra_id'][1])
                output = map(lambda e: '' if e is False or e is None else e, output)
                self.output_q.put(output)

            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



# -*- coding: utf-8 -*-
from datetime import datetime
import traceback
from libcnmc.utils import format_f
from libcnmc.core import StopMultiprocessBased


class F13bis(StopMultiprocessBased):
    def __init__(self, **kwargs):
        super(F13bis, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'F13 bis - PARCS'
        self.base_object = 'PARCS'

    def get_sequence(self):
        # Revisem que estigui actiu
        search_params = [('active', '!=', False)]
        return self.connection.GiscedataParcs.search(
            search_params, 0, 0, False, {'active_test': False})

    def get_subestacio(self, sub_id):
        """
        Returns the SE data
        :param sub_id: ID of SE
        :type sub_id: int
        :return: Node, Name, CINI and CT-ID of the SE
        :rtype: dict[str,str]
        """

        o = self.connection
        sub = o.GiscedataCtsSubestacions.read(
            sub_id, ['ct_id', 'cini', 'name', 'node_id']
        )
        ret = {
            "ct_id": sub['ct_id'][0],
            "cini": sub['cini'],
            "name": sub['name']
        }
        if 'node_id' in sub:
            ret["node"] = sub["node_id"][1]
        else:
            bloc_ids = o.GiscegisBlocsCtat.search([('ct', '=', ret["ct_id"])])
            node = ''
            if bloc_ids:
                bloc = o.GiscegisBlocsCtat.read(bloc_ids[0], ['node'])
                node = bloc['node'][1]
            ret["node"] = node
        return ret

    def get_tensio(self, parc_id):
        o = self.connection
        tensio_id = o.GiscedataParcs.read(
            parc_id, ['tensio_id'])['tensio_id'][0]
        return o.GiscedataTensionsTensio.read(tensio_id, ['tensio'])['tensio']

    def consumer(self):
        o = self.connection
        fields_to_read = [
            'id', 'subestacio_id', 'name', 'tipus', 'propietari', 'cini'
        ]
        while True:
            try:
                # generar linies
                item = self.input_q.get()
                if item == "STOP":
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                parc = o.GiscedataParcs.read(
                    item, fields_to_read
                )
                subestacio = self.get_subestacio(parc['subestacio_id'][0])
                o_subestacio = subestacio['name']
                o_parc = parc['name']
                o_node = subestacio['node']
                o_node = o_node.replace('*', '')
                o_cini = parc['cini']
                o_tipus = parc['tipus'] - 1
                tensio = self.get_tensio(parc['id'])
                o_tensio = format_f(
                    float(tensio) / 1000.0, decimals=3)
                o_prop = int(parc['propietari'])
                o_any = self.year
                insert = True
                if insert:
                    self.output_q.put([
                        o_subestacio,   # SUBESTACION
                        o_parc,         # PARQUE
                        o_node,         # NUDO
                        o_cini,         # CINI
                        o_tipus,        # TIPO PARQUE
                        o_tensio,       # TENSION DEL PARQUE
                        o_prop,         # PROPIEDAD
                        o_any           # AÑO INFORMACION
                    ])
                self.input_q.task_done()
            except Exception as e:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



# -*- coding: utf-8 -*-
import traceback
from datetime import datetime

from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import format_f


class FC1(StopMultiprocessBased):
    def __init__(self, **kwargs):
        super(FC1, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.report_name = (
            "Formulario C1: Gastos e ingresos operativos de la actividad de"
            " distribución"
        )
        self.base_object = 'C1'

    def get_sequence(self):
        prevision_ids = self.connection.model('cir8.2021.c1').search(
            [('year', '=', self.year)]
        )
        return prevision_ids

    def get_ine_ccaa(self, ccaa_id):
        O = self.connection
        codi_ccaa = O.model('res.comunitat_autonoma').read(ccaa_id, ['codi'])
        return codi_ccaa['codi']

    def consumer(self):
        O = self.connection
        fields_to_read = []
        while True:
            try:
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)

                c1 = O.model('cir8.2021.c1').read(item, fields_to_read)

                # CUENTA
                o_cuenta = ''
                if c1.get('cuenta', False):
                    o_cuenta = c1['cuenta']

                # CUENTA PGC
                o_cuenta_pgc = ''
                if c1.get('cuenta_pgc', False):
                    o_cuenta_pgc = c1['cuenta_pgc']

                # DESCRIPCION
                o_descripcion = ''
                if c1.get('descripcion', False):
                    o_descripcion = c1['descripcion']

                # CECO
                o_ceco = ''
                if c1.get('ceco', False):
                    o_ceco = c1['ceco']

                # COCO
                o_coco = ''
                if c1.get('coco', False):
                    o_coco = c1['coco']

                # PRESTACION SERVICIO
                o_prestacion_servicio = ''
                if c1.get('prestacion_servicio', False):
                    o_prestacion_servicio = c1['prestacion_servicio']

                # GASTO
                o_gasto = ''
                if c1.get('gasto', False):
                    o_gasto = format_f(c1['gasto'], decimals=2)

                # INGRESO
                o_ingreso = ''
                if c1.get('ingreso', False):
                    o_ingreso = format_f(c1['ingreso'], decimals=2)


                # CINI
                o_cini = ''
                if c1.get('cini', False):
                    o_cini = c1['cini']

                # TIPO COSTE
                o_tipo_coste = ''
                if c1.get('tipo_coste', False):
                    o_tipo_coste = c1['tipo_coste']

                # CCAA
                o_ccaa = ''
                if c1.get('ccaa', False):
                    o_ccaa = self.get_ine_ccaa(c1['ccaa'][0])

                # UNIDADES
                o_unidades = ''
                if c1.get('unidades', False):
                    o_unidades = c1['unidades']

                self.output_q.put([
                    o_cuenta,           # CUENTA
                    o_cuenta_pgc,           # CUENTA PGC
                    o_descripcion,          # DESCRIPCION
                    o_ceco,                 # CECO
                    o_coco,                 # COCO
                    o_prestacion_servicio,  # PRESTACION SERVICIO
                    o_gasto,                # GASTO
                    o_ingreso,              # INGRESO
                    o_cini,                 # CINI
                    o_tipo_coste,           # TIPO COSTE
                    o_ccaa,                 # CCAA
                    o_unidades,             # UNIDADES
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import traceback

from libcnmc.res_4667.utils import get_resum_any_id
from libcnmc.utils import format_f, get_codigo_ccaa
from libcnmc.core import MultiprocessBased


class PRO(MultiprocessBased):
    """
    Class to generate proyectos of 4667
    """

    def __init__(self, **kwargs):
        """
        Class constructor
        
        :param kwargs:
        :type kwargs: dict
        """

        self.year = kwargs.pop("year")

        self.proyectos = []

        self.vol_total_inv_prev_proy = {}
        self.ayudas_prv_proy = {}
        self.financiacion_prv_proy = {}
        self.vpi_retribuible_prv_proy = {}
        self.generated_proy = []
        self.ids = {}

        super(PRO, self).__init__(**kwargs)

    def get_sequence(self):
        """
        Generates the sequence of ids to make the report

        :return: List of ids
        :rtype: list
        """

        ids_resums = get_resum_any_id(self.connection, self.year)
        search_params = [("resums_inversio", "in", ids_resums)]

        model_proy = self.connection.GiscedataCnmcProjectes
        ids_proy = model_proy.search(search_params)
        for proy in model_proy.read(ids_proy, ["codi"]):
            self.proyectos.append(proy["codi"])
        self.proyectos = list(set(self.proyectos))

        self.vol_total_inv_prev_proy = dict.fromkeys(self.proyectos, 0.0)
        self.ayudas_prv_proy = dict.fromkeys(self.proyectos, 0.0)
        self.financiacion_prv_proy = dict.fromkeys(self.proyectos, 0.0)
        self.vpi_retribuible_prv_proy = dict.fromkeys(self.proyectos, 0.0)
        self.ids = dict.fromkeys(self.proyectos, 0.0)

        fields_read = [
            "vol_total_inv_prev_proy",
            "ayudas_prv_proy",
            "financiacion_prv_proy",
            "vpi_retribuible_prv_proy",
            "codi"
        ]
        for proy in model_proy.read(ids_proy, fields_read):
            self.vol_total_inv_prev_proy[proy["codi"]] += proy["vol_total_inv_prev_proy"]
            self.ayudas_prv_proy[proy["codi"]] += proy["ayudas_prv_proy"]
            self.financiacion_prv_proy[proy["codi"]] += proy["financiacion_prv_proy"]
            self.vpi_retribuible_prv_proy[proy["codi"]] += proy["vpi_retribuible_prv_proy"]
            self.ids[proy["codi"]] = proy["id"]

        return self.ids.values()

    def consumer(self):
        """
        Generates the line of the file

        :return: Line 
        :rtype: str
        """

        O = self.connection
        fields_to_read = []

        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)
                pro = O.GiscedataCnmcProjectes.read(item, fields_to_read)
                codigo = pro["codi"]
                ccaa = 0
                ccaa_2 = 0
                if pro["ccaa"]:
                    ccaa = pro["ccaa"][0]
                if pro["ccaa_2"]:
                    ccaa_2 = pro["ccaa_2"][0]

                output = [
                    pro["codi"],
                    pro["name"],
                    get_codigo_ccaa(O, ccaa),
                    get_codigo_ccaa(O, ccaa_2),
                    pro["memoria"].replace("\n", " ")[:300] or "",
                    format_f(self.vol_total_inv_prev_proy[codigo], 2) or "0.00",
                    format_f(self.ayudas_prv_proy[codigo], 2) or "0.00",
                    format_f(self.financiacion_prv_proy[codigo], 2) or "0.00",
                    format_f(self.vpi_retribuible_prv_proy[codigo], 2) or "0.00",
                    pro["estado"]
                ]


                self.output_q.put(output)
            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Generació dels CINIS de maquines
"""
from datetime import datetime
import traceback

from libcnmc.core import MultiprocessBased


class CINIMAQ(MultiprocessBased):
    def __init__(self, **kwargs):
        super(CINIMAQ, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'CINIMAQ'
        self.report_name = 'CNMC INVENTARI CINIMAQ'

    def get_sequence(self):
        search_params = [('id_estat.cnmc_inventari', '=', True)]
        return self.connection.GiscedataTransformadorTrafo.search(search_params)

    def consumer(self):
        O = self.connection
        fields_to_read = ['cini', 'tensio_primari_actual', 'tensio_b1',
                          'tensio_b2', 'potencia_nominal', 'reductor']
        while True:
            try:
                item = self.input_q.get()
                self.progress_q.put(item)
                cini = ''
                trafo = O.GiscedataTransformadorTrafo.read(
                    item, fields_to_read)

                if not trafo['cini']:
                    #Els cinis de màquines tots començen amb I27
                    cini_ = "I27"
                    #Per completar el tercer digit
                    codi3 = ''
                    try:
                        tensio_primari = int(trafo['tensio_primari_actual'])
                    except:
                        print "Error a {}, la tensio_primari no " \
                              "es correcte".format(item)
                    if tensio_primari <= 40000:
                        codi3 = '0'
                    elif 220000 <= tensio_primari < 400000:
                        codi3 = '1'
                    elif 110000 <= tensio_primari < 220000:
                        codi3 = '2'
                    elif 36000 <= tensio_primari < 110000:
                        codi3 = '3'
                    elif 1000 <= tensio_primari < 36000:
                        codi3 = '4'
                    #Per completar el quart digit
                    codi4 = ''
                    try:
                        #El camp tensio_b1 es un char s'ha de comprovar que es
                        #pot fer la conversio a enter
                        tensio_b1 = int(trafo['tensio_b1'])
                    except:
                        print "Error a {}, la tensio_b1 " \
                              "no es correcte".format(item)
                        continue
                    try:
                        tensio_b2 = int(trafo['tensio_b2'])
                    except:
                        print "Error a {}, la tensio_b2 " \
                              "no es correcte".format(item)
                        continue
                    if tensio_b1:
                        if 110000 <= tensio_b1 < 220000:
                            codi4 = '2'
                        elif 36000 <= tensio_b1 < 110000:
                            codi4 = '3'
                        elif 1000 <= tensio_b1 < 36000:
                            codi4 = '4'
                        elif tensio_b1 < 1000:
                            codi4 = '4'
                    elif tensio_b2:
                        if 110000 <= tensio_b2 < 220000:
                            codi4 = '2'
                        elif 36000 <= tensio_b2 < 110000:
                            codi4 = '3'
                        elif 1000 <= tensio_b2 < 36000:
                            codi4 = '4'
                        elif tensio_b2 < 1000:
                            codi4 = '4'
                    #Per completar el cinquè digit
                    codi5 = ''
                    if trafo['reductor']:
                        codi5 = '1'
                    else:
                        codi5 = '2'
                    #Per completar el sisè digit
                    codi6 = ''
                    try:
                        pot_nominal = int(trafo['potencia_nominal'])
                    except:
                        print "Error a {}, la potencia nominal" \
                              "no es correcte".format(item)
                        continue
                    if pot_nominal < 1000:
                        codi6 = 'A'
                    elif 1000 <= pot_nominal < 5000:
                        codi6 = 'B'
                    elif 5000 <= pot_nominal < 10000:
                        codi6 = 'C'
                    elif 10000 <= pot_nominal < 15000:
                        codi6 = 'D'
                    elif 15000 <= pot_nominal < 20000:
                        codi6 = 'E'
                    elif 20000 <= pot_nominal < 25000:
                        codi6 = 'F'
                    elif 25000 <= pot_nominal < 30000:
                        codi6 = 'G'
                    elif 30000 <= pot_nominal < 40000:
                        codi6 = 'H'
                    elif 40000 <= pot_nominal < 60000:
                        codi6 = 'I'
                    elif 60000 <= pot_nominal < 80000:
                        codi6 = 'J'
                    elif 80000 <= pot_nominal < 100000:
                        codi6 = 'K'
                    elif 100000 <= pot_nominal < 120000:
                        codi6 = 'L'
                    elif 120000 <= pot_nominal < 150000:
                        codi6 = 'M'
                    elif 150000 >= pot_nominal:
                        codi6 = 'N'

                    cini = cini_ + codi3 + codi4 + codi5 + codi6
                    #El write del cini en cada un dels trafos
                    O.GiscedataTransformadorTrafo.write([item], {'cini': cini})

                self.output_q.put(cini)
            except Exception:
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()
            finally:
                self.input_q.task_done()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Centres Transformadors
"""
from __future__ import absolute_import
from datetime import datetime
import traceback, psycopg2.extras
from libcnmc.utils import format_f, convert_srid, get_srid, get_norm_tension
from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import (
    format_f, get_id_municipi_from_company, get_forced_elements, adapt_diff, convert_srid, get_srid, format_f,
    convert_spanish_date, get_name_ti, format_f_6181, get_codi_actuacio, get_ine
)
from libcnmc.models import F5Res4666
from shapely import wkt


class FB5(StopMultiprocessBased):
    def __init__(self, **kwargs):
        super(FB5, self).__init__(**kwargs)
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'FB5 - TRAFOS-SE'
        self.base_object = 'TRAFOS'
        self.compare_field = '4666_entregada'

    def get_sequence(self):
        search_params = [
            ('reductor', '=', True),
            ('id_estat.cnmc_inventari', '=', True)
        ]
        return self.connection.GiscedataTransformadorTrafo.search(
            search_params
        )

    def get_node_vertex(self, ct_id):
        O = self.connection
        bloc = O.GiscegisBlocsCtat.search([('ct', '=', ct_id)])
        node = ''
        vertex = None
        if bloc:
            bloc = O.GiscegisBlocsCtat.read(bloc[0], ['node', 'vertex'])
            if bloc['node']:
                node = bloc['node'][1]
                if bloc['vertex']:
                    v = O.GiscegisVertex.read(bloc['vertex'][0], ['x', 'y'])
                    vertex = (round(v['x'], 3), round(v['y'], 3))
        return node, vertex

    def get_nodes(self, ct_id):
        o = self.connection
        ct = o.GiscedataCts.read(ct_id, ['node_id'])
        if ct.get("node_id"):
            o_node = ct["node_id"][1]
            node = o.GiscegisNodes.read(ct["node_id"][0], ["geom"])
            coords = wkt.loads(node["geom"]).coords[0]
            vertex = [coords[0], coords[1]]
        else:
            o_node, vertex = self.get_node_vertex(ct_id)
        o_node = o_node.replace('*', '')
        return o_node

    def consumer(self):
        o = self.connection
        fields_to_read = [
            'ct', 'name', 'cini', 'potencia_nominal', 'id_estat', 'node_id', 'data_pm', 'data_baixa',
            'tipus_instalacio_cnmc_id', 'node_baixa', 'model', self.compare_field
        ]
        fields_to_read_obra = [
            'subvenciones_europeas', 'subvenciones_nacionales', 'subvenciones_prtr', 'financiado',
            'fecha_aps', 'fecha_baja', 'causa_baja', 'cuenta_contable', 'im_ingenieria', 'im_materiales',
            'im_obracivil', 'im_trabajos', 'motivacion', 'tipo_inversion', 'valor_residual', 'identificador_baja'
        ]

        def get_inst_name(element_id):
            vals = self.connection.GiscedataTransformadorTrafo.read(
                element_id, ['name'])
            return vals['name']

        while True:
            try:
                # generar linies
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                trafo = o.GiscedataTransformadorTrafo.read(
                    item, fields_to_read
                )

                # MODEL
                if trafo['model']:
                    modelo = trafo['model']
                else:
                    modelo = ''

                # FECHA_APS
                if trafo['data_pm']:
                    data_pm_trafo = datetime.strptime(str(trafo['data_pm']),
                                                      '%Y-%m-%d')
                    data_pm = data_pm_trafo.strftime('%d/%m/%Y')

                # OBRES
                trafo_obra = ''
                obra_ti_trafo_obj = o.GiscedataProjecteObraTiTransformador
                obra_ti_ids = obra_ti_trafo_obj.search([('element_ti_id', '=', trafo['id'])])
                if obra_ti_ids:
                    for obra_ti_id in obra_ti_ids:
                        obra_id_data = obra_ti_trafo_obj.read(obra_ti_id, ['obra_id'])
                        obra_id = obra_id_data['obra_id']
                        # Filtre d'obres finalitzades
                        data_finalitzacio_data = o.GiscedataProjecteObra.read(obra_id[0], ['data_finalitzacio'])
                        if data_finalitzacio_data:
                            if data_finalitzacio_data.get('data_finalitzacio', False):
                                data_finalitzacio = data_finalitzacio_data['data_finalitzacio']

                                inici_any = '{}-01-01'.format(self.year)
                                fi_any = '{}-12-31'.format(self.year)
                                if obra_id and data_finalitzacio and inici_any <= data_finalitzacio <= fi_any:
                                    trafo_obra = o.GiscedataProjecteObraTiTransformador.read(obra_ti_id,
                                                                                             fields_to_read_obra)
                        if trafo_obra:
                            break

                tipo_inversion = ''
                #CAMPS OBRES
                if trafo_obra != '':
                    obra_year = data_finalitzacio.split('-')[0]
                    data_pm_year = data_pm.split('/')[2]
                    if trafo_obra['tipo_inversion'] != '0' and obra_year != data_pm_year:
                        data_ip = convert_spanish_date(data_finalitzacio)
                    else:
                        data_ip = ''
                    identificador_baja = (
                        get_inst_name(trafo_obra['identificador_baja']) if trafo_obra['identificador_baja'] else ''
                    )
                    subvenciones_europeas = format_f_6181(trafo_obra['subvenciones_europeas'] or 0.0, float_type='euro')
                    subvenciones_nacionales = format_f_6181(trafo_obra['subvenciones_nacionales'] or 0.0, float_type='euro')
                    subvenciones_prtr = format_f_6181(trafo_obra['subvenciones_prtr'] or 0.0, float_type='euro')
                    im_ingenieria = format_f_6181(trafo_obra['im_ingenieria'] or 0.0, float_type='euro')
                    im_materiales = format_f_6181(trafo_obra['im_materiales'] or 0.0, float_type='euro')
                    im_obracivil = format_f_6181(trafo_obra['im_obracivil'] or 0.0, float_type='euro')
                    im_trabajos = format_f_6181(trafo_obra['im_trabajos'] or 0.0, float_type='euro')
                    im_construccion = str(format_f(
                        float(im_materiales.replace(",", ".")) + float(im_obracivil.replace(",", "."))
                    , 2)).replace(".", ",")
                    tipo_inversion = trafo_obra['tipo_inversion'] or ''
                    valor_auditado = str(
                        float(im_construccion.replace(",", ".")) +
                        float(im_ingenieria.replace(",", ".")) + float(im_trabajos.replace(",", "."))
                    ).replace(".", ",")
                    valor_residual = format_f(trafo_obra.get('valor_residual', 0.0), 2)
                    cuenta_contable = trafo_obra['cuenta_contable']
                    financiado = format_f(trafo_obra.get('financiado', 0.0), 2)
                    motivacion = get_codi_actuacio(o, trafo_obra['motivacion'] and trafo_obra['motivacion'][0])
                else:
                    data_ip = ''
                    identificador_baja = ''
                    im_ingenieria = ''
                    im_construccion = ''
                    im_trabajos = ''
                    subvenciones_europeas = ''
                    subvenciones_nacionales = ''
                    subvenciones_prtr = ''
                    valor_auditado = ''
                    motivacion = ''
                    cuenta_contable = ''
                    avifauna = ''
                    financiado = ''
                    valor_residual = ''

                # Si la data APS es igual a l'any de la generació del fitxer,
                # la data APS sortirà en blanc
                if data_ip:
                    data_ip = '' if data_pm and int(data_pm.split('/')[2]) == int(data_ip.split('/')[2]) \
                        else data_ip

                o_subestacio = trafo['ct'][1]
                o_maquina = trafo['name']
                o_cini = trafo['cini']
                o_pot_maquina = format_f(
                    float(trafo['potencia_nominal']) / 1000.0, decimals=3)
                o_node = trafo['node_id'][1]

                if trafo['node_baixa']:
                    o_node_baixa = trafo['node_baixa'][1]
                else:
                    o_node_baixa = o_node

                if o_cini:
                    if o_cini[1] == '2' and o_cini[2] == '4' and o_cini[4] == '0' and o_cini[7] == '1':
                        o_node_baixa = o_node

                #FECHA_BAJA, CAUSA_BAJA

                data_pm_limit = '{0}-01-01'.format(self.year + 1)
                if trafo['data_baixa']:
                    if trafo['data_baixa'] < data_pm_limit:
                        tmp_date = datetime.strptime(
                            trafo['data_baixa'], '%Y-%m-%d')
                        fecha_baja = tmp_date.strftime('%d/%m/%Y')
                        if int(fecha_baja.split("/")[2]) - int(data_pm.split("/")[2]) >= 40:
                            if identificador_baja != '':
                                causa_baja = 1
                            else:
                                causa_baja = 2
                        else:
                            causa_baja = 3
                    else:
                        fecha_baja = ''
                        causa_baja = 0;
                else:
                    fecha_baja = ''
                    causa_baja = 0;

                #CODIGO CCUU
                id_ti = trafo['tipus_instalacio_cnmc_id'][0]
                ti = o.GiscedataTipusInstallacio.read(
                    id_ti,
                    ['name'])['name']

                # ESTADO
                if trafo[self.compare_field]:
                    last_data = trafo[self.compare_field]
                    entregada = F5Res4666(**last_data)
                    actual = F5Res4666(
                        trafo['name'],
                        trafo['cini'],
                        '',
                        ti,
                        '',
                        '',
                        '',
                        '',
                        data_pm,
                        fecha_baja,
                        '',
                        0
                    )
                    if entregada == actual and fecha_baja == '':
                        estado = '0'
                        if trafo_obra:
                            estado = '1'
                    else:
                        self.output_m.put("{} {}".format(trafo["name"], adapt_diff(actual.diff(entregada))))
                        estado = '1'
                else:
                    estado = '2'

                if modelo == 'M':
                    estado = ''
                    data_pm = ''

                if fecha_baja:
                    tipo_inversion = ''

                # L'any 2022 no es declaren subvencions PRTR
                subvenciones_prtr = ''

                if causa_baja == '0':
                    fecha_baja = ''

                if modelo == 'E':
                    tipo_inversion = '0'
                    estado = '1'

                self.output_q.put([
                    o_maquina,              # IDENTIFICADOR_MAQUINA
                    o_cini,                 # CINI
                    ti,                     # CCUU
                    o_node,             #NUDO_ALTA
                    o_node_baixa,       #NUDO_BAJA
                    o_pot_maquina,       # POTENCIA MAQUINA
                    estado,                 # ESTADO
                    modelo,             #MODELO
                    data_pm,               #FECHA_APS
                    fecha_baja,            #FECHA_BAJA
                    causa_baja,            #CAUSA_BAJA
                    data_ip,            #FECHA_IP
                    tipo_inversion,     #TIPO_INVERSION
                    im_ingenieria,      #IM_TRAMITES
                    im_construccion,    #IM_CONSTRUCCION
                    im_trabajos,        #IM_TRABAJOS
                    valor_residual,     #VALOR_RESIDUAL
                    subvenciones_europeas,  #SUBVENCIONES_EUROPEAS
                    subvenciones_nacionales, #SUBVENCIONES_NACIONALES
                    subvenciones_prtr,  #SUBVENCIONES_PRTR
                    valor_auditado,          #VALOR_AUDITADO
                    financiado,         #FINANCIADO
                    cuenta_contable,       #CUENTA
                    motivacion,         #MOTIVACION
                    identificador_baja,     #IDENTIFICADOR BAJA
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC - TOVP
"""
from __future__ import absolute_import
from datetime import datetime
import traceback, psycopg2.extras
from libcnmc.core import StopMultiprocessBased
from libcnmc.utils import format_f, get_id_municipi_from_company, get_ine


class FC2(StopMultiprocessBased):

    """
    Class that generates the CT file of the 4666
    """
    def __init__(self, **kwargs):
        super(FC2, self).__init__(**kwargs)
        self.codi_r1 = kwargs.pop('codi_r1')
        self.year = kwargs.pop('year', datetime.now().year - 1)
        self.report_name = 'FC2 - TOVP'
        self.base_object = 'TOVP'
        self.compare_field = '4666_entregada'

    def get_sequence(self):
        c2_ids = self.connection.model('cir8.2021.c2').search([('year', '=', self.year)])
        return c2_ids

    def get_ine(self, municipi_id):
        O = self.connection
        muni = O.ResMunicipi.read(municipi_id, ['ine', 'dc'])
        return get_ine(O, muni['ine'])

    def consumer(self):
        O = self.connection

        fields_to_read = ['municipio', 'tovp']

        while True:
            try:
                item = self.input_q.get()
                if item == 'STOP':
                    self.input_q.task_done()
                    break
                self.progress_q.put(item)
                c2 = O.model('cir8.2021.c2').read(item, fields_to_read)

                # MUNICIPIO I PROVINCIA
                municipio = ''
                provincia = ''
                if c2.get('municipio', False):
                    provincia, municipio = self.get_ine(c2['municipio'][0])

                # CCAA
                fun_ccaa = O.ResComunitat_autonoma.get_ccaa_from_municipi
                if c2['municipio']:
                    id_municipi = c2['municipio'][0]
                else:
                    id_municipi = get_id_municipi_from_company(O)

                comunitat_codi = ''
                if id_municipi:
                    id_comunitat = fun_ccaa(id_municipi)
                    comunitat_vals = O.ResComunitat_autonoma.read(
                        id_comunitat[0], ['codi'])
                    if comunitat_vals:
                        comunitat_codi = comunitat_vals['codi']

                # TOVP
                tovp = '0,000'
                if c2.get('tovp', False):
                    tovp = c2['tovp']

                self.output_q.put([
                    municipio,                                  # MUNICIPIO
                    provincia,                                  # PROVINCIA
                    comunitat_codi,                             # CCAA
                    format_f(tovp, decimals=2),                 # TOVP
                ])
                self.input_q.task_done()
            except Exception:
                self.input_q.task_done()
                traceback.print_exc()
                if self.raven:
                    self.raven.captureException()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
INVENTARI DE CNMC Inventari
"""
import csv
import os
import traceback
from chardet import detect

from libcnmc.core import cnmc_inventari

try:
    from raven import Client
except:
    Client = None

QUIET = False
from libcnmc import VERSION


class INV():
    def __init__(self, **kwargs):
        self.codi_r1 = kwargs.pop('codi_r1')
        self.base_object = 'Inventari'
        self.report_name = 'CNMC INVENTARI XML'
        self.liniesat = kwargs.pop('liniesat')
        self.liniesbt = kwargs.pop('liniesbt')
        self.subestacions = kwargs.pop('subestacions')
        self.posicions = kwargs.pop('posicions')
        self.maquinas = kwargs.pop('maquinas')
        self.despatxos = kwargs.pop('despatxos')
        self.fiabilidad = kwargs.pop('fiabilidad')
        self.transformacion = kwargs.pop('transformacion')
        self.file_out = kwargs.pop('output')
        self.pla_inversions_xml = cnmc_inventari.Empresa(self.codi_r1)
        if 'SENTRY_DSN' in os.environ and Client:
            self.raven = Client()
            self.raven.tags_context({'version': VERSION})
        else:
            self.raven = None

    def check_encoding(self):
        """
        Checks the encoding of the files
        
        :return: None
        :rtype: None
        """

        input_files = ['liniesat', 'liniesbt', 'subestacions', 'posicions',
                       'maquinas', 'despatxos', 'fiabilidad', 'transformacion']
        for input_f in input_files:
            with open(getattr(self, input_f), 'r') as f:
                result = detect(f.read())
                if result['encoding'] not in ('utf-8', 'ascii'):
                    raise Exception('File: %s is not in UTF-8.' % input_f)

    def open_csv_file(self, csv_file):
        """
        Opens a CSV file 
        
        :param csv_file: URL of the file
        :type csv_file: str
        :return: csv reader
        :rtype: csv.reader
        """

        reader = csv.reader(open(csv_file), delimiter=';')
        return reader

    def tractar_linies(self, arxiucsvlinies):
        reader = self.open_csv_file(arxiucsvlinies)
        for row in reader:
            if not row:
                continue
            linia = cnmc_inventari.Linea()

            identificador = row[0]
            cini = row[1]
            origen = row[2]
            destino = row[3]
            codigo_tipo_linea = row[4]
            codigo_ccaa_1 = row[5]
            codigo_ccaa_2 = row[6]
            participacion = row[7]
            fecha_aps = row[8]
            fecha_baja = row[9]
            numero_circuitos = row[10]
            numero_conductores = row[11]
            longitud = row[12]
            seccion = row[13]
            capacidad = row[14]

            linia.feed({
                'identificador': '%s' % identificador.strip(),
                'cini': '%s' % cini.strip(),
                'origen': '%s' % origen.strip(),
                'destino': '%s' % destino.strip(),
                'codigo_tipo_linea': '%s' % codigo_tipo_linea,
                'codigo_ccaa_1': '%s' % codigo_ccaa_1,
                'codigo_ccaa_2': '%s' % codigo_ccaa_2,
                'participacion': '%s' % participacion,
                'fecha_aps': '%s' % fecha_aps.strip(),
                'fecha_baja': '%s' % fecha_baja.strip(),
                'numero_circuitos': '%s' % numero_circuitos,
                'numero_conductores': '%s' % numero_conductores,
                'longitud': '%s' % longitud,
                'seccion': '%s' % seccion,
                'capacidad': '%s' % capacidad
            })
            self.pla_inversions_xml.linea.append(linia)

    def tractar_linies_at(self):
        self.tractar_linies(self.liniesat)

    def tractar_linies_bt(self):
        self.tractar_linies(self.liniesbt)

    def tractar_sub(self, arxiucsvsub):
        reader = self.open_csv_file(arxiucsvsub)
        for row in reader:
            if not row:
                continue
            sub = cnmc_inventari.Subestacion()

            identificador = row[0]
            cini = row[1]
            denominacion = row[2]
            codigo_tipo_posicion = row[3]
            codigo_ccaa = row[4]
            participacion = row[5]
            fecha_aps = row[6]
            fecha_baja = row[7]
            posiciones = row[8]

            sub.feed({
                'identificador': '%s' % identificador.strip(),
                'cini': '%s' % cini.strip(),
                'denominacion': '%s' % denominacion.strip(),
                'codigo_tipo_posicion': '%s' % codigo_tipo_posicion,
                'codigo_ccaa': '%s' % codigo_ccaa,
                'participacion': '%s' % participacion,
                'fecha_aps': '%s' % fecha_aps.strip(),
                'fecha_baja': '%s' % fecha_baja.strip(),
                'posiciones': '%s' % posiciones,
            })
            self.pla_inversions_xml.linea.append(sub)

    def tractar_subestacions(self):
        self.tractar_sub(self.subestacions)

    def tractar_pos(self, arxiucsvpos):
        reader = self.open_csv_file(arxiucsvpos)
        for row in reader:
            if not row:
                continue
            pos = cnmc_inventari.Posicion()

            identificador = row[0]
            cini = row[1]
            denominacion = row[2]
            codigo_tipo_posicion = row[3]
            codigo_ccaa = row[4]
            participacion = row[5]
            fecha_aps = row[6]
            fecha_baja = row[7]

            pos.feed({
                'identificador': '%s' % identificador.strip(),
                'cini': '%s' % cini.strip(),
                'denominacion': '%s' % denominacion.strip(),
                'codigo_tipo_posicion': '%s' % codigo_tipo_posicion,
                'codigo_ccaa': '%s' % codigo_ccaa,
                'participacion': '%s' % participacion,
                'fecha_aps': '%s' % fecha_aps.strip(),
                'fecha_baja': '%s' % fecha_baja.strip(),
            })
            self.pla_inversions_xml.linea.append(pos)

    def tractar_posicions(self):
        self.tractar_pos(self.posicions)

    def tractar_maq(self, arxiucsvmaq):
        reader = self.open_csv_file(arxiucsvmaq)
        for row in reader:
            if not row:
                continue
            maq = cnmc_inventari.Maquina()

            identificador = row[0]
            cini = row[1]
            denominacion = row[2]
            codigo_tipo_maquina = row[3]
            codigo_zona = row[4]
            codigo_ccaa = row[5]
            participacion = row[6]
            fecha_aps = row[7]
            fecha_baja = row[8]
            capacidad = row[9]

            maq.feed({
                'identificador': '%s' % identificador.strip(),
                'cini': '%s' % cini.strip(),
                'denominacion': '%s' % denominacion.strip(),
                'codigo_tipo_maquina': '%s' % codigo_tipo_maquina,
                'codigo_zona': '%s' % codigo_zona,
                'codigo_ccaa': '%s' % codigo_ccaa,
                'participacion': '%s' % participacion,
                'fecha_aps': '%s' % fecha_aps.strip(),
                'fecha_baja': '%s' % fecha_baja.strip(),
                'capacidad': '%s' % capacidad,
            })
            self.pla_inversions_xml.linea.append(maq)

    def tractar_maquines(self):
        self.tractar_maq(self.maquinas)

    def tractar_desp(self, arxiucsvdesp):
        reader = self.open_csv_file(arxiucsvdesp)
        for row in reader:
            if not row:
                continue
            despt = cnmc_inventari.Despacho()

            identificador = row[0]
            cini = row[1]
            denominacion = row[2]
            anyo_ps = row[3]
            vai = row[4]

            despt.feed({
                'identificador': '%s' % identificador.strip(),
                'cini': '%s' % cini.strip(),
                'denominacion': '%s' % denominacion.strip(),
                'anyo_ps': '%s' % anyo_ps,
                'vai': '%s' % vai,
            })
            self.pla_inversions_xml.linea.append(despt)

    def tractar_despatxos(self):
        self.tractar_desp(self.despatxos)

    def tractar_fia(self, arxiucsvfia):
        reader = self.open_csv_file(arxiucsvfia)
        for row in reader:
            if not row:
                continue
            despt = cnmc_inventari.Fiabilidad()

            identificador = row[0]
            cini = row[1]
            denominacion = row[2]
            codigo_tipo_inst = row[3]
            codigo_ccaa = row[4]
            fecha_aps = row[5]
            fecha_baja = row[6]

            despt.feed({
                'identificador': '%s' % identificador.strip(),
                'cini': '%s' % cini.strip(),
                'denominacion': '%s' % denominacion.strip(),
                'codigo_tipo_inst': '%s' % codigo_tipo_inst.strip(),
                'codigo_ccaa': '%s' % codigo_ccaa,
                'fecha_aps': '%s' % fecha_aps.strip(),
                'fecha_baja': '%s' % fecha_baja.strip(),
            })
            self.pla_inversions_xml.linea.append(despt)

    def tractar_equips_fiabilitat(self):
        self.tractar_fia(self.fiabilidad)

    def tractar_trans(self, arxiucsvtrans):
        reader = self.open_csv_file(arxiucsvtrans)
        for row in reader:
            if not row:
                continue
            trans = cnmc_inventari.Transformacion()

            identificador = row[0]
            cini = row[1]
            denominacion = row[2]
            codigo_tipo_ct = row[3]
            codigo_ccaa = row[4]
            participacion = row[5]
            fecha_aps = row[6]
            fecha_baja = row[7]

            trans.feed({
                'identificador': '%s' % identificador.strip(),
                'cini': '%s' % cini.strip(),
                'denominacion': '%s' % denominacion.strip(),
                'codigo_tipo_ct': '%s' % codigo_tipo_ct.strip(),
                'codigo_ccaa': '%s' % codigo_ccaa,
                'participacion': '%s' % participacion,
                'fecha_aps': '%s' % fecha_aps.strip(),
                'fecha_baja': '%s' % fecha_baja.strip(),
            })
            self.pla_inversions_xml.linea.append(trans)

    def tractar_centres_transformadors(self):
        self.tractar_trans(self.transformacion)

    def escriure_xml(self, arxiuxml):
        self.pla_inversions_xml.set_xml_encoding('iso-8859-1')
        self.pla_inversions_xml.build_tree()
        arxiuxml.write(str(self.pla_inversions_xml))

    def calc(self):
        try:
            self.check_encoding()
            #Obro l'arxiu XML a emplenar
            arxiuxml = open(self.file_out, 'wb')

            # Carrega dels fitxers CSV LAT
            self.tractar_linies_at()

            # Carrega dels fitxers CSV LBT
            self.tractar_linies_bt()

            # Carrega dels fitxers CSV subestacions
            self.tractar_subestacions()

            # Carrega dels fitxers CSV posicions
            self.tractar_posicions()

            # Carrega dels fitxers CSV maquines
            self.tractar_maquines()

            # Carrega dels fitxers CSV despatxos
            self.tractar_despatxos()

            # Carrega dels fitxers CSV fiabilitat
            self.tractar_equips_fiabilitat()

            # Carrega dels fitxers CSV transformadors
            self.tractar_centres_transformadors()

            # Escrivim l'arxiu XML
            self.escriure_xml(arxiuxml)
        except Exception:
            traceback.print_exc()
            if self.raven:
                self.raven.captureException()
            raise